from collections.abc import Callable, Iterable, Iterator
from typing import Any, TypedDict, TypeVar
from typing_extensions import TypeAlias

_T = TypeVar("_T")
_Callback: TypeAlias = Callable[[str, _Result], object]

class _Result(TypedDict):
    nmap: _ResultNmap
    scan: dict[str, PortScannerHostDict]

class _ResultNmap(TypedDict):
    command_line: str
    scaninfo: _ResultNmapInfo
    scanstats: _ResultNampStats

class _ResultNmapInfo(TypedDict, total=False):
    error: str
    warning: str
    protocol: _ResultNampInfoProtocol

class _ResultNampInfoProtocol(TypedDict):
    method: str
    services: str

class _ResultNampStats(TypedDict):
    timestr: str
    elapsed: str
    uphosts: str
    downhosts: str
    totalhosts: str

class _ResulHostUptime(TypedDict):
    seconds: str
    lastboot: str

class _ResultHostNames(TypedDict):
    type: str
    name: str

class _ResultHostPort(TypedDict):
    conf: str
    cpe: str
    extrainfo: str
    name: str
    product: str
    reason: str
    state: str
    version: str

__last_modification__: str
__author__: str
__version__: str

class PortScanner:
    def __init__(
        self,
        nmap_search_path: Iterable[str] = ("nmap", "/usr/bin/nmap", "/usr/local/bin/nmap", "/sw/bin/nmap", "/opt/local/bin/nmap"),
    ) -> None: ...
    def get_nmap_last_output(self) -> str: ...
    def nmap_version(self) -> tuple[int, int]: ...
    def listscan(self, hosts: str = "127.0.0.1") -> list[str]: ...
    def scan(
        self, hosts: str = "127.0.0.1", ports: str | None = None, arguments: str = "-sV", sudo: bool = False, timeout: int = 0
    ) -> _Result: ...
    def analyse_nmap_xml_scan(
        self,
        nmap_xml_output: str | None = None,
        nmap_err: str = "",
        nmap_err_keep_trace: str = "",
        nmap_warn_keep_trace: str = "",
    ) -> _Result: ...
    def __getitem__(self, host: str) -> PortScannerHostDict: ...
    def all_hosts(self) -> list[str]: ...
    def command_line(self) -> str: ...
    def scaninfo(self) -> _ResultNmapInfo: ...
    def scanstats(self) -> _ResultNampStats: ...
    def has_host(self, host: str) -> bool: ...
    def csv(self) -> str: ...

def __scan_progressive__(
    self: object, hosts: str, ports: str, arguments: str, callback: _Callback | None, sudo: bool, timeout: int
) -> None: ...

class PortScannerAsync:
    def __init__(self) -> None: ...
    def __del__(self) -> None: ...
    def scan(
        self,
        hosts: str = "127.0.0.1",
        ports: str | None = None,
        arguments: str = "-sV",
        callback: _Callback | None = None,
        sudo: bool = False,
        timeout: int = 0,
    ) -> None: ...
    def stop(self) -> None: ...
    def wait(self, timeout: int | None = None) -> None: ...
    def still_scanning(self) -> bool: ...

class PortScannerYield(PortScannerAsync):
    def __init__(self) -> None: ...
    def scan(  # type: ignore[override]
        self, hosts: str = "127.0.0.1", ports: str | None = None, arguments: str = "-sV", sudo: bool = False, timeout: int = 0
    ) -> Iterator[tuple[str, _Result]]: ...
    def stop(self) -> None: ...
    def wait(self, timeout: int | None = None) -> None: ...
    def still_scanning(self) -> None: ...  # type: ignore[override]

class PortScannerHostDict(dict[str, Any]):
    def hostnames(self) -> list[_ResultHostNames]: ...
    def hostname(self) -> str: ...
    def state(self) -> str: ...
    def uptime(self) -> _ResulHostUptime: ...
    def all_protocols(self) -> list[str]: ...
    def all_tcp(self) -> list[int]: ...
    def has_tcp(self, port: int) -> bool: ...
    def tcp(self, port: int) -> _ResultHostPort: ...
    def all_udp(self) -> list[int]: ...
    def has_udp(self, port: int) -> bool: ...
    def udp(self, port: int) -> _ResultHostPort: ...
    def all_ip(self) -> list[int]: ...
    def has_ip(self, port: int) -> bool: ...
    def ip(self, port: int) -> _ResultHostPort: ...
    def all_sctp(self) -> list[int]: ...
    def has_sctp(self, port: int) -> bool: ...
    def sctp(self, port: int) -> _ResultHostPort: ...

class PortScannerError(Exception):
    value: str
    def __init__(self, value: str) -> None: ...

class PortScannerTimeout(PortScannerError): ...

def convert_nmap_output_to_encoding(value: _T, code: str = "ascii") -> _T: ...



import datetime as dt
from _typeshed import SupportsAllComparisons
from typing import Any, Generic, TypeVar, overload
from typing_extensions import Self

from psycopg2._psycopg import _type, connection, cursor

_T_co = TypeVar("_T_co", covariant=True)

class Range(Generic[_T_co]):
    def __init__(
        self, lower: _T_co | None = None, upper: _T_co | None = None, bounds: str = "[)", empty: bool = False
    ) -> None: ...
    @property
    def lower(self) -> _T_co | None: ...
    @property
    def upper(self) -> _T_co | None: ...
    @property
    def isempty(self) -> bool: ...
    @property
    def lower_inf(self) -> bool: ...
    @property
    def upper_inf(self) -> bool: ...
    @property
    def lower_inc(self) -> bool: ...
    @property
    def upper_inc(self) -> bool: ...
    def __contains__(self, x: SupportsAllComparisons) -> bool: ...
    def __bool__(self) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: Range[_T_co]) -> bool: ...
    def __le__(self, other: Range[_T_co]) -> bool: ...
    def __gt__(self, other: Range[_T_co]) -> bool: ...
    def __ge__(self, other: Range[_T_co]) -> bool: ...

def register_range(
    pgrange: str, pyrange: str | type[Range[Any]], conn_or_curs: connection | cursor, globally: bool = False
) -> RangeCaster: ...

class RangeAdapter:
    name: str | None = None
    adapted: Range[Any]
    def __init__(self, adapted: Range[Any]) -> None: ...
    def __conform__(self, proto) -> Self | None: ...
    def prepare(self, conn: connection | None) -> None: ...
    def getquoted(self) -> bytes: ...

class RangeCaster:
    adapter: type[RangeAdapter]
    range: type[Range[Any]]
    subtype_oid: int
    typecaster: _type
    array_typecaster: _type | None
    def __init__(
        self,
        pgrange: str | type[RangeAdapter],
        pyrange: str | type[Range[Any]],
        oid: int,
        subtype_oid: int,
        array_oid: int | None = None,
    ) -> None: ...
    @overload
    def parse(self, s: None, cur: cursor | None = None) -> None: ...
    @overload
    def parse(self, s: str, cur: cursor | None = None) -> Range[Any]: ...
    @overload
    def parse(self, s: str | None, cur: cursor | None = None) -> Range[Any] | None: ...

class NumericRange(Range[float]): ...
class DateRange(Range[dt.date]): ...
class DateTimeRange(Range[dt.datetime]): ...
class DateTimeTZRange(Range[dt.datetime]): ...

class NumberRangeAdapter(RangeAdapter):
    def getquoted(self) -> bytes: ...

int4range_caster: RangeCaster
int8range_caster: RangeCaster
numrange_caster: RangeCaster
daterange_caster: RangeCaster
tsrange_caster: RangeCaster
tstzrange_caster: RangeCaster



from _typeshed import Incomplete
from typing import ClassVar

from .core import UnitedStates

class DistrictOfColumbia(UnitedStates):
    include_inauguration_day: ClassVar[bool]
    FIXED_HOLIDAYS: Incomplete



import tkinter
from _typeshed import Incomplete

from ._widget import ThemedWidget

class ThemedTk(tkinter.Tk, ThemedWidget):
    def __init__(
        self,
        # non-keyword-only args copied from tkinter.Tk
        screenName: str | None = ...,
        baseName: str | None = ...,
        className: str = ...,
        useTk: bool = ...,
        sync: bool = ...,
        use: str | None = ...,
        *,
        theme: str | None = ...,
        # fonts argument does nothing
        toplevel: bool | None = ...,
        themebg: bool | None = ...,
        background: bool | None = ...,  # old alias for themebg
        gif_override: bool = ...,
    ) -> None: ...
    def set_theme(self, theme_name, toplevel: bool | None = None, themebg: bool | None = None) -> None: ...
    # TODO: currently no good way to say "use the same big list of kwargs as parent class but also add these"
    def config(self, kw: Incomplete | None = None, **kwargs): ...  # type: ignore[override]
    def cget(self, k): ...
    def configure(self, kw: Incomplete | None = None, **kwargs): ...  # type: ignore[override]
    def __getitem__(self, k): ...
    def __setitem__(self, k, v) -> None: ...



from enum import IntEnum, IntFlag

class ApplyLocation(IntEnum):
    WORKDIR: int
    INDEX: int
    BOTH: int

class AttrCheck(IntFlag):
    FILE_THEN_INDEX: int
    INDEX_THEN_FILE: int
    INDEX_ONLY: int
    NO_SYSTEM: int
    INCLUDE_HEAD: int
    INCLUDE_COMMIT: int

class BlameFlag(IntFlag):
    NORMAL: int
    TRACK_COPIES_SAME_FILE: int
    TRACK_COPIES_SAME_COMMIT_MOVES: int
    TRACK_COPIES_SAME_COMMIT_COPIES: int
    TRACK_COPIES_ANY_COMMIT_COPIES: int
    FIRST_PARENT: int
    USE_MAILMAP: int
    IGNORE_WHITESPACE: int

class BlobFilter(IntFlag):
    CHECK_FOR_BINARY: int
    NO_SYSTEM_ATTRIBUTES: int
    ATTRIBUTES_FROM_HEAD: int
    ATTRIBUTES_FROM_COMMIT: int

class BranchType(IntFlag):
    LOCAL: int
    REMOTE: int
    ALL: int

class CheckoutNotify(IntFlag):
    NONE: int
    CONFLICT: int
    DIRTY: int
    UPDATED: int
    UNTRACKED: int
    IGNORED: int
    ALL: int

class CheckoutStrategy(IntFlag):
    NONE: int
    SAFE: int
    FORCE: int
    RECREATE_MISSING: int
    ALLOW_CONFLICTS: int
    REMOVE_UNTRACKED: int
    REMOVE_IGNORED: int
    UPDATE_ONLY: int
    DONT_UPDATE_INDEX: int
    NO_REFRESH: int
    SKIP_UNMERGED: int
    USE_OURS: int
    USE_THEIRS: int
    DISABLE_PATHSPEC_MATCH: int
    SKIP_LOCKED_DIRECTORIES: int
    DONT_OVERWRITE_IGNORED: int
    CONFLICT_STYLE_MERGE: int
    CONFLICT_STYLE_DIFF3: int
    DONT_REMOVE_EXISTING: int
    DONT_WRITE_INDEX: int
    DRY_RUN: int
    CONFLICT_STYLE_ZDIFF3: int

class ConfigLevel(IntEnum):
    PROGRAMDATA: int
    SYSTEM: int
    XDG: int
    GLOBAL: int
    LOCAL: int
    APP: int
    HIGHEST_LEVEL: int

class CredentialType(IntFlag):
    USERPASS_PLAINTEXT: int
    SSH_KEY: int
    SSH_CUSTOM: int
    DEFAULT: int
    SSH_INTERACTIVE: int
    USERNAME: int
    SSH_MEMORY: int

class DeltaStatus(IntEnum):
    UNMODIFIED: int
    ADDED: int
    DELETED: int
    MODIFIED: int
    RENAMED: int
    COPIED: int
    IGNORED: int
    UNTRACKED: int
    TYPECHANGE: int
    UNREADABLE: int
    CONFLICTED: int

class DescribeStrategy(IntEnum):
    DEFAULT: int
    TAGS: int
    ALL: int

class DiffFind(IntFlag):
    FIND_BY_CONFIG: int
    FIND_RENAMES: int
    FIND_RENAMES_FROM_REWRITES: int
    FIND_COPIES: int
    FIND_COPIES_FROM_UNMODIFIED: int
    FIND_REWRITES: int
    BREAK_REWRITES: int
    FIND_AND_BREAK_REWRITES: int
    FIND_FOR_UNTRACKED: int
    FIND_ALL: int
    FIND_IGNORE_LEADING_WHITESPACE: int
    FIND_IGNORE_WHITESPACE: int
    FIND_DONT_IGNORE_WHITESPACE: int
    FIND_EXACT_MATCH_ONLY: int
    BREAK_REWRITES_FOR_RENAMES_ONLY: int
    FIND_REMOVE_UNMODIFIED: int

class DiffFlag(IntFlag):
    BINARY: int
    NOT_BINARY: int
    VALID_ID: int
    EXISTS: int
    VALID_SIZE: int

class DiffOption(IntFlag):
    NORMAL: int
    REVERSE: int
    INCLUDE_IGNORED: int
    RECURSE_IGNORED_DIRS: int
    INCLUDE_UNTRACKED: int
    RECURSE_UNTRACKED_DIRS: int
    INCLUDE_UNMODIFIED: int
    INCLUDE_TYPECHANGE: int
    INCLUDE_TYPECHANGE_TREES: int
    IGNORE_FILEMODE: int
    IGNORE_SUBMODULES: int
    IGNORE_CASE: int
    INCLUDE_CASECHANGE: int
    DISABLE_PATHSPEC_MATCH: int
    SKIP_BINARY_CHECK: int
    ENABLE_FAST_UNTRACKED_DIRS: int
    UPDATE_INDEX: int
    INCLUDE_UNREADABLE: int
    INCLUDE_UNREADABLE_AS_UNTRACKED: int
    INDENT_HEURISTIC: int
    IGNORE_BLANK_LINES: int
    FORCE_TEXT: int
    FORCE_BINARY: int
    IGNORE_WHITESPACE: int
    IGNORE_WHITESPACE_CHANGE: int
    IGNORE_WHITESPACE_EOL: int
    SHOW_UNTRACKED_CONTENT: int
    SHOW_UNMODIFIED: int
    PATIENCE: int
    MINIMAL: int
    SHOW_BINARY: int

class DiffStatsFormat(IntFlag):
    NONE: int
    FULL: int
    SHORT: int
    NUMBER: int
    INCLUDE_SUMMARY: int

class Feature(IntFlag):
    THREADS: int
    HTTPS: int
    SSH: int
    NSEC: int

class FetchPrune(IntEnum):
    UNSPECIFIED: int
    PRUNE: int
    NO_PRUNE: int

class FileMode(IntFlag):
    UNREADABLE: int
    TREE: int
    BLOB: int
    BLOB_EXECUTABLE: int
    LINK: int
    COMMIT: int

class FileStatus(IntFlag):
    CURRENT: int
    INDEX_NEW: int
    INDEX_MODIFIED: int
    INDEX_DELETED: int
    INDEX_RENAMED: int
    INDEX_TYPECHANGE: int
    WT_NEW: int
    WT_MODIFIED: int
    WT_DELETED: int
    WT_TYPECHANGE: int
    WT_RENAMED: int
    WT_UNREADABLE: int
    IGNORED: int
    CONFLICTED: int

class FilterFlag(IntFlag):
    DEFAULT: int
    ALLOW_UNSAFE: int
    NO_SYSTEM_ATTRIBUTES: int
    ATTRIBUTES_FROM_HEAD: int
    ATTRIBUTES_FROM_COMMIT: int

class FilterMode(IntEnum):
    TO_WORKTREE: int
    SMUDGE: int
    TO_ODB: int
    CLEAN: int

class MergeAnalysis(IntFlag):
    NONE: int
    NORMAL: int
    UP_TO_DATE: int
    FASTFORWARD: int
    UNBORN: int

class MergeFavor(IntEnum):
    NORMAL: int
    OURS: int
    THEIRS: int
    UNION: int

class MergeFileFlag(IntFlag):
    DEFAULT: int
    STYLE_MERGE: int
    STYLE_DIFF3: int
    SIMPLIFY_ALNUM: int
    IGNORE_WHITESPACE: int
    IGNORE_WHITESPACE_CHANGE: int
    IGNORE_WHITESPACE_EOL: int
    DIFF_PATIENCE: int
    DIFF_MINIMAL: int
    STYLE_ZDIFF3: int
    ACCEPT_CONFLICTS: int

class MergeFlag(IntFlag):
    FIND_RENAMES: int
    FAIL_ON_CONFLICT: int
    SKIP_REUC: int
    NO_RECURSIVE: int
    VIRTUAL_BASE: int

class MergePreference(IntFlag):
    NONE: int
    NO_FASTFORWARD: int
    FASTFORWARD_ONLY: int

class ObjectType(IntEnum):
    ANY: int
    INVALID: int
    COMMIT: int
    TREE: int
    BLOB: int
    TAG: int
    OFS_DELTA: int
    REF_DELTA: int

class Option(IntEnum):
    GET_MWINDOW_SIZE: int
    SET_MWINDOW_SIZE: int
    GET_MWINDOW_MAPPED_LIMIT: int
    SET_MWINDOW_MAPPED_LIMIT: int
    GET_SEARCH_PATH: int
    SET_SEARCH_PATH: int
    SET_CACHE_OBJECT_LIMIT: int
    SET_CACHE_MAX_SIZE: int
    ENABLE_CACHING: int
    GET_CACHED_MEMORY: int
    GET_TEMPLATE_PATH: int
    SET_TEMPLATE_PATH: int
    SET_SSL_CERT_LOCATIONS: int
    SET_USER_AGENT: int
    ENABLE_STRICT_OBJECT_CREATION: int
    ENABLE_STRICT_SYMBOLIC_REF_CREATION: int
    SET_SSL_CIPHERS: int
    GET_USER_AGENT: int
    ENABLE_OFS_DELTA: int
    ENABLE_FSYNC_GITDIR: int
    GET_WINDOWS_SHAREMODE: int
    SET_WINDOWS_SHAREMODE: int
    ENABLE_STRICT_HASH_VERIFICATION: int
    SET_ALLOCATOR: int
    ENABLE_UNSAVED_INDEX_SAFETY: int
    GET_PACK_MAX_OBJECTS: int
    SET_PACK_MAX_OBJECTS: int
    DISABLE_PACK_KEEP_FILE_CHECKS: int
    GET_OWNER_VALIDATION: int
    SET_OWNER_VALIDATION: int

class ReferenceFilter(IntEnum):
    ALL: int
    BRANCHES: int
    TAGS: int

class ReferenceType(IntFlag):
    INVALID: int
    DIRECT: int
    SYMBOLIC: int
    ALL: int
    OID: int
    LISTALL: int

class RepositoryInitFlag(IntFlag):
    BARE: int
    NO_REINIT: int
    NO_DOTGIT_DIR: int
    MKDIR: int
    MKPATH: int
    EXTERNAL_TEMPLATE: int
    RELATIVE_GITLINK: int

class RepositoryInitMode(IntEnum):
    SHARED_UMASK: int
    SHARED_GROUP: int
    SHARED_ALL: int

class RepositoryOpenFlag(IntFlag):
    DEFAULT: int
    NO_SEARCH: int
    CROSS_FS: int
    BARE: int
    NO_DOTGIT: int
    FROM_ENV: int

class RepositoryState(IntEnum):
    NONE: int
    MERGE: int
    REVERT: int
    REVERT_SEQUENCE: int
    CHERRYPICK: int
    CHERRYPICK_SEQUENCE: int
    BISECT: int
    REBASE: int
    REBASE_INTERACTIVE: int
    REBASE_MERGE: int
    APPLY_MAILBOX: int
    APPLY_MAILBOX_OR_REBASE: int

class ResetMode(IntEnum):
    SOFT: int
    MIXED: int
    HARD: int

class RevSpecFlag(IntFlag):
    SINGLE: int
    RANGE: int
    MERGE_BASE: int

class SortMode(IntFlag):
    NONE: int
    TOPOLOGICAL: int
    TIME: int
    REVERSE: int

class StashApplyProgress(IntEnum):
    NONE: int
    LOADING_STASH: int
    ANALYZE_INDEX: int
    ANALYZE_MODIFIED: int
    ANALYZE_UNTRACKED: int
    CHECKOUT_UNTRACKED: int
    CHECKOUT_MODIFIED: int
    DONE: int

class SubmoduleIgnore(IntEnum):
    UNSPECIFIED: int
    NONE: int
    UNTRACKED: int
    DIRTY: int
    ALL: int

class SubmoduleStatus(IntFlag):
    IN_HEAD: int
    IN_INDEX: int
    IN_CONFIG: int
    IN_WD: int
    INDEX_ADDED: int
    INDEX_DELETED: int
    INDEX_MODIFIED: int
    WD_UNINITIALIZED: int
    WD_ADDED: int
    WD_DELETED: int
    WD_MODIFIED: int
    WD_INDEX_MODIFIED: int
    WD_WD_MODIFIED: int
    WD_UNTRACKED: int



from _typeshed import Incomplete
from collections.abc import Callable, Iterable
from typing import Any, Literal, overload
from typing_extensions import TypeAlias

import pandas as pd
from matplotlib.axes import Axes
from matplotlib.typing import ColorType
from numpy.typing import NDArray

from .axisgrid import FacetGrid
from .utils import _Palette, _Seed

__all__ = ["lmplot", "regplot", "residplot"]

_Vector: TypeAlias = list[Incomplete] | pd.Series[Incomplete] | pd.Index[Incomplete] | NDArray[Incomplete]

def lmplot(
    data: pd.DataFrame,
    *,
    x: str | None = None,
    y: str | None = None,
    hue: str | None = None,
    col: str | None = None,
    row: str | None = None,
    palette: _Palette | None = None,
    col_wrap: int | None = None,
    height: float = 5,
    aspect: float = 1,
    markers: str = "o",
    sharex: bool | Literal["col", "row"] | None = None,  # deprecated
    sharey: bool | Literal["col", "row"] | None = None,  # deprecated
    hue_order: Iterable[str] | None = None,
    col_order: Iterable[str] | None = None,
    row_order: Iterable[str] | None = None,
    legend: bool = True,
    legend_out: bool | None = None,  # deprecated
    x_estimator: Callable[[Incomplete], Incomplete] | None = None,
    x_bins: int | _Vector | None = None,
    x_ci: Literal["ci", "sd"] | int | None = "ci",
    scatter: bool = True,
    fit_reg: bool = True,
    ci: int | None = 95,
    n_boot: int = 1000,
    units: str | None = None,
    seed: _Seed | None = None,
    order: int = 1,
    logistic: bool = False,
    lowess: bool = False,
    robust: bool = False,
    logx: bool = False,
    x_partial: str | None = None,
    y_partial: str | None = None,
    truncate: bool = True,
    x_jitter: float | None = None,
    y_jitter: float | None = None,
    scatter_kws: dict[str, Any] | None = None,
    line_kws: dict[str, Any] | None = None,
    facet_kws: dict[str, Any] | None = None,
) -> FacetGrid: ...
@overload
def regplot(
    data: None = None,
    *,
    x: _Vector | None = None,
    y: _Vector | None = None,
    x_estimator: Callable[[Incomplete], Incomplete] | None = None,
    x_bins: int | _Vector | None = None,
    x_ci: Literal["ci", "sd"] | int | None = "ci",
    scatter: bool = True,
    fit_reg: bool = True,
    ci: int | None = 95,
    n_boot: int = 1000,
    units: _Vector | None = None,
    seed: _Seed | None = None,
    order: int = 1,
    logistic: bool = False,
    lowess: bool = False,
    robust: bool = False,
    logx: bool = False,
    x_partial: _Vector | None = None,
    y_partial: _Vector | None = None,
    truncate: bool = True,
    dropna: bool = True,
    x_jitter: float | None = None,
    y_jitter: float | None = None,
    label: str | None = None,
    color: ColorType | None = None,
    marker: str = "o",
    scatter_kws: dict[str, Any] | None = None,
    line_kws: dict[str, Any] | None = None,
    ax: Axes | None = None,
) -> Axes: ...
@overload
def regplot(
    data: pd.DataFrame,
    *,
    x: str | _Vector | None = None,
    y: str | _Vector | None = None,
    x_estimator: Callable[[Incomplete], Incomplete] | None = None,
    x_bins: int | _Vector | None = None,
    x_ci: Literal["ci", "sd"] | int | None = "ci",
    scatter: bool = True,
    fit_reg: bool = True,
    ci: int | None = 95,
    n_boot: int = 1000,
    units: str | _Vector | None = None,
    seed: _Seed | None = None,
    order: int = 1,
    logistic: bool = False,
    lowess: bool = False,
    robust: bool = False,
    logx: bool = False,
    x_partial: str | _Vector | None = None,
    y_partial: str | _Vector | None = None,
    truncate: bool = True,
    dropna: bool = True,
    x_jitter: float | None = None,
    y_jitter: float | None = None,
    label: str | None = None,
    color: ColorType | None = None,
    marker: str = "o",
    scatter_kws: dict[str, Any] | None = None,
    line_kws: dict[str, Any] | None = None,
    ax: Axes | None = None,
) -> Axes: ...
@overload
def residplot(
    data: None = None,
    *,
    x: _Vector | None = None,
    y: _Vector | None = None,
    x_partial: _Vector | None = None,
    y_partial: _Vector | None = None,
    lowess: bool = False,
    order: int = 1,
    robust: bool = False,
    dropna: bool = True,
    label: str | None = None,
    color: ColorType | None = None,
    scatter_kws: dict[str, Any] | None = None,
    line_kws: dict[str, Any] | None = None,
    ax: Axes | None = None,
) -> Axes: ...
@overload
def residplot(
    data: pd.DataFrame,
    *,
    x: str | _Vector | None = None,
    y: str | _Vector | None = None,
    x_partial: str | _Vector | None = None,
    y_partial: str | _Vector | None = None,
    lowess: bool = False,
    order: int = 1,
    robust: bool = False,
    dropna: bool = True,
    label: str | None = None,
    color: ColorType | None = None,
    scatter_kws: dict[str, Any] | None = None,
    line_kws: dict[str, Any] | None = None,
    ax: Axes | None = None,
) -> Axes: ...



from _typeshed import Incomplete

from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Audit(SystemBackendMixin):
    def list_enabled_audit_devices(self): ...
    def enable_audit_device(
        self,
        device_type,
        description: Incomplete | None = None,
        options: Incomplete | None = None,
        path: Incomplete | None = None,
        local: Incomplete | None = None,
    ): ...
    def disable_audit_device(self, path): ...
    def calculate_hash(self, path, input_to_hash): ...



import sys
from _typeshed import ReadableBuffer
from typing_extensions import Never

from serial.serialutil import SerialBase

class PlatformSpecificBase:
    BAUDRATE_CONSTANTS: dict[int, int]
    def set_low_latency_mode(self, low_latency_settings: bool) -> None: ...

CMSPAR: int
if sys.platform == "linux":
    TCGETS2: int
    TCSETS2: int
    BOTHER: int
    TIOCGRS485: int
    TIOCSRS485: int
    SER_RS485_ENABLED: int
    SER_RS485_RTS_ON_SEND: int
    SER_RS485_RTS_AFTER_SEND: int
    SER_RS485_RX_DURING_TX: int

    class PlatformSpecific(PlatformSpecificBase): ...

elif sys.platform == "cygwin":
    class PlatformSpecific(PlatformSpecificBase): ...

elif sys.platform == "darwin":
    IOSSIOSPEED: int

    class PlatformSpecific(PlatformSpecificBase):
        osx_version: list[str]
        TIOCSBRK: int
        TIOCCBRK: int

else:
    class PlatformSpecific(PlatformSpecificBase): ...

TIOCMGET: int
TIOCMBIS: int
TIOCMBIC: int
TIOCMSET: int
TIOCM_DTR: int
TIOCM_RTS: int
TIOCM_CTS: int
TIOCM_CAR: int
TIOCM_RNG: int
TIOCM_DSR: int
TIOCM_CD: int
TIOCM_RI: int
TIOCINQ: int
TIOCOUTQ: int
TIOCM_zero_str: bytes
TIOCM_RTS_str: bytes
TIOCM_DTR_str: bytes
TIOCSBRK: int
TIOCCBRK: int

class Serial(SerialBase, PlatformSpecific):
    fd: int | None
    pipe_abort_read_w: int | None
    pipe_abort_read_r: int | None
    pipe_abort_write_w: int | None
    pipe_abort_write_r: int | None
    def open(self) -> None: ...
    @property
    def in_waiting(self) -> int: ...
    def read(self, size: int = 1) -> bytes: ...
    def cancel_read(self) -> None: ...
    def cancel_write(self) -> None: ...
    def write(self, b: ReadableBuffer, /) -> int | None: ...
    def reset_input_buffer(self) -> None: ...
    def reset_output_buffer(self) -> None: ...
    def send_break(self, duration: float = ...) -> None: ...
    @property
    def cts(self) -> bool: ...
    @property
    def dsr(self) -> bool: ...
    @property
    def ri(self) -> bool: ...
    @property
    def cd(self) -> bool: ...
    @property
    def out_waiting(self) -> int: ...
    def set_input_flow_control(self, enable: bool = ...) -> None: ...
    def set_output_flow_control(self, enable: bool = ...) -> None: ...
    def nonblocking(self) -> None: ...

class PosixPollSerial(Serial): ...

class VTIMESerial(Serial):
    @property
    def cancel_read(self) -> Never: ...



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import Any, ClassVar
from typing_extensions import TypeAlias

__version__: str

_Argv: TypeAlias = Iterable[str] | str

class DocoptLanguageError(Exception): ...

class DocoptExit(SystemExit):
    usage: ClassVar[str]
    def __init__(self, message: str = "") -> None: ...

def printable_usage(doc: str) -> str: ...
def docopt(
    doc: str, argv: _Argv | None = ..., help: bool = ..., version: Incomplete | None = ..., options_first: bool = ...
) -> dict[str, Any]: ...  # Really should be dict[str, str | bool]



from _typeshed import Incomplete

from stripe import api_requestor as api_requestor
from stripe.api_resources.abstract.api_resource import APIResource as APIResource

class UsageRecord(APIResource):
    OBJECT_NAME: str
    @classmethod
    def create(
        cls,
        api_key: Incomplete | None = None,
        idempotency_key: str | None = None,
        stripe_version: Incomplete | None = None,
        stripe_account: Incomplete | None = None,
        **params,
    ): ...



class WhatThePatchException(Exception): ...

class HunkException(WhatThePatchException):
    hunk: int | None
    def __init__(self, msg: str, hunk: int | None = None) -> None: ...

class ApplyException(WhatThePatchException): ...

class SubprocessException(ApplyException):
    code: int
    def __init__(self, msg: str, code: int) -> None: ...

class HunkApplyException(HunkException, ApplyException, ValueError): ...
class ParseException(HunkException, ValueError): ...



from _typeshed import (
    FileDescriptorOrPath,
    Incomplete,
    OpenBinaryMode,
    OpenBinaryModeReading,
    OpenBinaryModeUpdating,
    OpenBinaryModeWriting,
    OpenTextMode,
)
from asyncio import AbstractEventLoop
from collections.abc import Callable
from typing import Literal, overload
from typing_extensions import TypeAlias

from ..base import AiofilesContextManager
from .binary import AsyncBufferedIOBase, AsyncBufferedReader, AsyncFileIO, AsyncIndirectBufferedIOBase, _UnknownAsyncBinaryIO
from .text import AsyncTextIndirectIOWrapper, AsyncTextIOWrapper

_Opener: TypeAlias = Callable[[str, int], int]

# Text mode: always returns AsyncTextIOWrapper
@overload
def open(
    file: FileDescriptorOrPath,
    mode: OpenTextMode = "r",
    buffering: int = -1,
    encoding: str | None = None,
    errors: str | None = None,
    newline: str | None = None,
    closefd: bool = True,
    opener: _Opener | None = None,
    *,
    loop: AbstractEventLoop | None = None,
    executor: Incomplete | None = None,
) -> AiofilesContextManager[None, None, AsyncTextIOWrapper]: ...

# Unbuffered binary: returns a FileIO
@overload
def open(
    file: FileDescriptorOrPath,
    mode: OpenBinaryMode,
    buffering: Literal[0],
    encoding: None = None,
    errors: None = None,
    newline: None = None,
    closefd: bool = True,
    opener: _Opener | None = None,
    *,
    loop: AbstractEventLoop | None = None,
    executor: Incomplete | None = None,
) -> AiofilesContextManager[None, None, AsyncFileIO]: ...

# Buffered binary reading/updating: AsyncBufferedReader
@overload
def open(
    file: FileDescriptorOrPath,
    mode: OpenBinaryModeReading | OpenBinaryModeUpdating,
    buffering: Literal[-1, 1] = -1,
    encoding: None = None,
    errors: None = None,
    newline: None = None,
    closefd: bool = True,
    opener: _Opener | None = None,
    *,
    loop: AbstractEventLoop | None = None,
    executor: Incomplete | None = None,
) -> AiofilesContextManager[None, None, AsyncBufferedReader]: ...

# Buffered binary writing: AsyncBufferedIOBase
@overload
def open(
    file: FileDescriptorOrPath,
    mode: OpenBinaryModeWriting,
    buffering: Literal[-1, 1] = -1,
    encoding: None = None,
    errors: None = None,
    newline: None = None,
    closefd: bool = True,
    opener: _Opener | None = None,
    *,
    loop: AbstractEventLoop | None = None,
    executor: Incomplete | None = None,
) -> AiofilesContextManager[None, None, AsyncBufferedIOBase]: ...

# Buffering cannot be determined: fall back to _UnknownAsyncBinaryIO
@overload
def open(
    file: FileDescriptorOrPath,
    mode: OpenBinaryMode,
    buffering: int = -1,
    encoding: None = None,
    errors: None = None,
    newline: None = None,
    closefd: bool = True,
    opener: _Opener | None = None,
    *,
    loop: AbstractEventLoop | None = None,
    executor: Incomplete | None = None,
) -> AiofilesContextManager[None, None, _UnknownAsyncBinaryIO]: ...

stdin: AsyncTextIndirectIOWrapper
stdout: AsyncTextIndirectIOWrapper
stderr: AsyncTextIndirectIOWrapper
stdin_bytes: AsyncIndirectBufferedIOBase
stdout_bytes: AsyncIndirectBufferedIOBase
stderr_bytes: AsyncIndirectBufferedIOBase



from _typeshed import Incomplete

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class ActiveDirectory(VaultApiBase):
    def configure(
        self,
        binddn: Incomplete | None = None,
        bindpass: Incomplete | None = None,
        url: Incomplete | None = None,
        userdn: Incomplete | None = None,
        upndomain: Incomplete | None = None,
        ttl: Incomplete | None = None,
        max_ttl: Incomplete | None = None,
        mount_point="ad",
        *args,
        **kwargs,
    ): ...
    def read_config(self, mount_point="ad"): ...
    def create_or_update_role(
        self, name, service_account_name: Incomplete | None = None, ttl: Incomplete | None = None, mount_point="ad"
    ): ...
    def read_role(self, name, mount_point="ad"): ...
    def list_roles(self, mount_point="ad"): ...
    def delete_role(self, name, mount_point="ad"): ...
    def generate_credentials(self, name, mount_point="ad"): ...



from collections.abc import Generator
from typing import ClassVar, Literal

from .. import fixer_base

MAPPING: dict[str, dict[str, str]]
LOOKUP: dict[tuple[str, str], str]

def alternates(members): ...
def build_pattern() -> Generator[str, None, None]: ...

class FixRenames(fixer_base.BaseFix):
    BM_compatible: ClassVar[Literal[True]]
    order: ClassVar[Literal["pre"]]
    PATTERN: ClassVar[str]
    def match(self, node): ...
    def transform(self, node, results) -> None: ...



from .ElementTree import (
    XML as XML,
    ParseError as ParseError,
    XMLParse as XMLParse,
    XMLParser as XMLParser,
    XMLTreeBuilder as XMLTreeBuilder,
    fromstring as fromstring,
    iterparse as iterparse,
    parse as parse,
    tostring as tostring,
)

__all__ = ["ParseError", "XML", "XMLParse", "XMLParser", "XMLTreeBuilder", "fromstring", "iterparse", "parse", "tostring"]



from _typeshed import Incomplete
from xmlrpc.client import ExpatParser

__origin__: str
MAX_DATA: int = 31457280

def defused_gzip_decode(data, limit: int | None = None): ...

# Couldn't type this as a class deriving from gzip.GzipFile
# since overwriting `read` method does not define an optional argument
# for size when the underlying class does.
DefusedGzipDecodedResponse = Incomplete

class DefusedExpatParser(ExpatParser):
    forbid_dtd: bool
    forbid_entities: bool
    forbid_external: bool
    def __init__(self, target, forbid_dtd: bool = False, forbid_entities: bool = True, forbid_external: bool = True) -> None: ...
    def defused_start_doctype_decl(self, name, sysid, pubid, has_internal_subset) -> None: ...
    def defused_entity_decl(self, name, is_parameter_entity, value, base, sysid, pubid, notation_name) -> None: ...
    def defused_unparsed_entity_decl(self, name, base, sysid, pubid, notation_name) -> None: ...
    def defused_external_entity_ref_handler(self, context, base, sysid, pubid) -> None: ...

def monkey_patch() -> None: ...
def unmonkey_patch() -> None: ...



from _typeshed import Incomplete

hour: Incomplete
minute: Incomplete
second: Incomplete
newton_precision: Incomplete

def calculate_equinoxes(year, timezone: str = ...): ...
def get_current_longitude(current_date, earth, sun): ...
def newton(f, x0, x1, precision=..., **func_kwargs): ...
def newton_angle_function(t, ts, target_angle, body1, body2): ...
def solar_term(year, degrees, timezone: str = ...): ...



from _typeshed import Incomplete
from collections.abc import Callable
from threading import Thread
from typing import Any, Protocol, TypeVar, overload
from typing_extensions import ParamSpec, TypeAlias

from flask import Flask
from flask.testing import FlaskClient

from .namespace import Namespace
from .test_client import SocketIOTestClient

_P = ParamSpec("_P")
_R_co = TypeVar("_R_co", covariant=True)
_ExceptionHandler: TypeAlias = Callable[[BaseException], _R_co]
_Handler: TypeAlias = Callable[_P, _R_co]

class _HandlerDecorator(Protocol):
    def __call__(self, handler: _Handler[_P, _R_co]) -> _Handler[_P, _R_co]: ...

class _ExceptionHandlerDecorator(Protocol):
    def __call__(self, exception_handler: _ExceptionHandler[_R_co]) -> _ExceptionHandler[_R_co]: ...

class SocketIO:
    # Many instance attributes are deliberately not included here,
    # as the maintainer of Flask-SocketIO considers them private, internal details:
    # https://github.com/python/typeshed/pull/10735#discussion_r1330768869
    def __init__(
        self,
        app: Flask | None = None,
        *,
        # SocketIO options
        manage_session: bool = True,
        message_queue: str | None = None,
        channel: str = "flask-socketio",
        path: str = "socket.io",
        resource: str = "socket.io",
        **kwargs,  # TODO: Socket.IO server options, Engine.IO server config
    ) -> None: ...
    def init_app(
        self,
        app: Flask,
        *,
        # SocketIO options
        manage_session: bool = True,
        message_queue: str | None = None,
        channel: str = "flask-socketio",
        path: str = "socket.io",
        resource: str = "socket.io",
        **kwargs,  # TODO: Socket.IO server options, Engine.IO server config: ...
    ) -> None: ...
    def on(self, message: str, namespace: str | None = None) -> _HandlerDecorator: ...
    def on_error(self, namespace: str | None = None) -> _ExceptionHandlerDecorator: ...
    def on_error_default(self, exception_handler: _ExceptionHandler[_R_co]) -> _ExceptionHandler[_R_co]: ...
    def on_event(self, message: str, handler: _Handler[[Incomplete], object], namespace: str | None = None) -> None: ...
    @overload
    def event(self, event_handler: _Handler[_P, _R_co], /) -> _Handler[_P, _R_co]: ...
    @overload
    def event(self, namespace: str | None = None, *args, **kwargs) -> _HandlerDecorator: ...
    def on_namespace(self, namespace_handler: Namespace) -> None: ...
    def emit(
        self,
        event: str,
        *args,
        namespace: str = "/",  # / is the default (global) namespace
        to: str | None = None,
        include_self: bool = True,
        skip_sid: str | list[str] | None = None,
        callback: Callable[..., Incomplete] | None = None,
    ) -> None: ...
    def call(
        self,
        event: str,
        *args,
        namespace: str = "/",  # / is the default (global) namespace
        to: str | None = None,
        timeout: int = 60,  # seconds
        ignore_queue: bool = False,
    ): ...
    def send(
        self,
        data: Any,
        json: bool = False,
        namespace: str | None = None,
        to: str | None = None,
        callback: Callable[..., Incomplete] | None = None,
        include_self: bool = True,
        skip_sid: list[str] | str | None = None,
        **kwargs,
    ) -> None: ...
    def close_room(self, room: str, namespace: str | None = None) -> None: ...
    def run(
        self,
        app,
        host: str | None = None,
        port: int | None = None,
        *,
        debug: bool = True,
        use_reloader: bool,
        reloader_options: dict[str, Incomplete] = {},
        log_output: bool,
        allow_unsafe_werkzeug: bool = False,
        **kwargs,
    ) -> None: ...
    def stop(self) -> None: ...
    def start_background_task(self, target: Callable[_P, None], *args: _P.args, **kwargs: _P.kwargs) -> Thread: ...
    def sleep(self, seconds: int = 0): ...
    def test_client(
        self,
        app: Flask,
        namespace: str | None = None,
        query_string: str | None = None,
        headers: dict[str, Incomplete] | None = None,
        auth: dict[str, Incomplete] | None = None,
        flask_test_client: FlaskClient | None = None,
    ) -> SocketIOTestClient: ...

def emit(
    event,
    *args,
    namespace: str = "/",  # / is the default (global) namespace
    to: str | None = None,
    include_self: bool = True,
    skip_sid: str | list[str] | None = None,
    callback: Callable[..., Incomplete] | None = None,
    broadcast: bool = False,
) -> None: ...
def send(message: str, **kwargs) -> None: ...
def join_room(room, sid: str | None = None, namespace: str | None = None) -> None: ...
def leave_room(room, sid: str | None = None, namespace: str | None = None) -> None: ...
def close_room(room, namespace: str | None = None) -> None: ...
def rooms(sid: str | None = None, namespace: str | None = None) -> list[str]: ...
def disconnect(sid: str | None = None, namespace: str | None = None, silent: bool = False) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable, Generator
from typing import Literal, overload

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatch

@_dispatch
def bridges(G: Graph[_Node], root: _Node | None = None) -> Generator[_Node, None, None]: ...
@_dispatch
def has_bridges(G: Graph[_Node], root: Incomplete | None = None) -> bool: ...
@overload
def local_bridges(
    G: Graph[_Node], with_span: Literal[False], weight: str | Callable[[_Node], float] | None = None
) -> Generator[tuple[_Node, _Node], None, None]: ...
@overload
def local_bridges(
    G: Graph[_Node], with_span: Literal[True] = True, weight: str | Callable[[_Node], float] | None = None
) -> Generator[tuple[_Node, _Node, int], None, None]: ...



from _typeshed import Incomplete
from typing import ClassVar, Literal

from .ImageFile import ImageFile

MODES: Incomplete

class TgaImageFile(ImageFile):
    format: ClassVar[Literal["TGA"]]
    format_description: ClassVar[str]

SAVE: Incomplete



from _typeshed import Incomplete

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Azure(VaultApiBase):
    def configure(
        self,
        subscription_id,
        tenant_id,
        client_id: Incomplete | None = None,
        client_secret: Incomplete | None = None,
        environment: Incomplete | None = None,
        mount_point="azure",
    ): ...
    def read_config(self, mount_point="azure"): ...
    def delete_config(self, mount_point="azure"): ...
    def create_or_update_role(
        self, name, azure_roles, ttl: Incomplete | None = None, max_ttl: Incomplete | None = None, mount_point="azure"
    ): ...
    def list_roles(self, mount_point="azure"): ...
    def generate_credentials(self, name, mount_point="azure"): ...



from _collections_abc import Generator, dict_keys
from _typeshed import Incomplete, ReadableBuffer
from types import TracebackType
from typing import Literal
from typing_extensions import Self, TypeAlias

from pyasn1.type.base import Asn1Item

from .pooling import ServerPool
from .server import Server

SASL_AVAILABLE_MECHANISMS: Incomplete
CLIENT_STRATEGIES: Incomplete

_ServerSequence: TypeAlias = (
    set[Server] | list[Server] | tuple[Server, ...] | Generator[Server, None, None] | dict_keys[Server, Incomplete]
)

class Connection:
    connection_lock: Incomplete
    last_error: str
    strategy_type: Incomplete
    user: Incomplete
    password: Incomplete
    authentication: Incomplete
    version: Incomplete
    auto_referrals: Incomplete
    request: Incomplete
    response: Incomplete | None
    result: Incomplete
    bound: bool
    listening: bool
    closed: bool
    auto_bind: Incomplete
    sasl_mechanism: Incomplete
    sasl_credentials: Incomplete
    socket: Incomplete
    tls_started: bool
    sasl_in_progress: bool
    read_only: Incomplete
    lazy: Incomplete
    pool_name: Incomplete
    pool_size: int | None
    cred_store: Incomplete
    pool_lifetime: Incomplete
    pool_keepalive: Incomplete
    starting_tls: bool
    check_names: Incomplete
    raise_exceptions: Incomplete
    auto_range: Incomplete
    extend: Incomplete
    fast_decoder: Incomplete
    receive_timeout: Incomplete
    empty_attributes: Incomplete
    use_referral_cache: Incomplete
    auto_escape: Incomplete
    auto_encode: Incomplete
    source_address: Incomplete
    source_port_list: Incomplete
    server_pool: Incomplete | None
    server: Incomplete
    strategy: Incomplete
    send: Incomplete
    open: Incomplete
    get_response: Incomplete
    post_send_single_response: Incomplete
    post_send_search: Incomplete
    def __init__(
        self,
        server: Server | str | _ServerSequence | ServerPool,
        user: str | None = None,
        password: str | None = None,
        auto_bind: Literal["DEFAULT", "NONE", "NO_TLS", "TLS_BEFORE_BIND", "TLS_AFTER_BIND"] | bool = "DEFAULT",
        version: int = 3,
        authentication: Literal["ANONYMOUS", "SIMPLE", "SASL", "NTLM"] | None = None,
        client_strategy: Literal[
            "SYNC",
            "SAFE_RESTARTABLE",
            "SAFE_SYNC",
            "ASYNC",
            "LDIF",
            "RESTARTABLE",
            "REUSABLE",
            "MOCK_SYNC",
            "MOCK_ASYNC",
            "ASYNC_STREAM",
        ] = "SYNC",
        auto_referrals: bool = True,
        auto_range: bool = True,
        sasl_mechanism: str | None = None,
        sasl_credentials: Incomplete | None = None,
        check_names: bool = True,
        collect_usage: bool = False,
        read_only: bool = False,
        lazy: bool = False,
        raise_exceptions: bool = False,
        pool_name: str | None = None,
        pool_size: int | None = None,
        pool_lifetime: int | None = None,
        cred_store: Incomplete | None = None,
        fast_decoder: bool = True,
        receive_timeout: Incomplete | None = None,
        return_empty_attributes: bool = True,
        use_referral_cache: bool = False,
        auto_escape: bool = True,
        auto_encode: bool = True,
        pool_keepalive: Incomplete | None = None,
        source_address: str | None = None,
        source_port: int | None = None,
        source_port_list: Incomplete | None = None,
    ) -> None: ...
    def repr_with_sensitive_data_stripped(self): ...
    @property
    def stream(self): ...
    @stream.setter
    def stream(self, value) -> None: ...
    @property
    def usage(self): ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> Literal[False] | None: ...
    def bind(self, read_server_info: bool = True, controls: Incomplete | None = None): ...
    def rebind(
        self,
        user: Incomplete | None = None,
        password: Incomplete | None = None,
        authentication: Incomplete | None = None,
        sasl_mechanism: Incomplete | None = None,
        sasl_credentials: Incomplete | None = None,
        read_server_info: bool = True,
        controls: Incomplete | None = None,
    ): ...
    def unbind(self, controls: Incomplete | None = None): ...
    def search(
        self,
        search_base: str,
        search_filter: str,
        search_scope: Literal["BASE", "LEVEL", "SUBTREE"] = "SUBTREE",
        dereference_aliases: Literal["NEVER", "SEARCH", "FINDING_BASE", "ALWAYS"] = "ALWAYS",
        attributes: Incomplete | None = None,
        size_limit: int = 0,
        time_limit: int = 0,
        types_only: bool = False,
        get_operational_attributes: bool = False,
        controls: Incomplete | None = None,
        paged_size: int | None = None,
        paged_criticality: bool = False,
        paged_cookie: str | bytes | None = None,
        auto_escape: bool | None = None,
    ): ...
    def compare(self, dn, attribute, value, controls: Incomplete | None = None): ...
    def add(
        self, dn, object_class: Incomplete | None = None, attributes: Incomplete | None = None, controls: Incomplete | None = None
    ): ...
    def delete(self, dn, controls: Incomplete | None = None): ...
    def modify(self, dn, changes, controls: Incomplete | None = None): ...
    def modify_dn(
        self,
        dn,
        relative_dn,
        delete_old_dn: bool = True,
        new_superior: Incomplete | None = None,
        controls: Incomplete | None = None,
    ): ...
    def abandon(self, message_id, controls: Incomplete | None = None): ...
    def extended(
        self,
        request_name,
        request_value: Asn1Item | ReadableBuffer | None = None,
        controls: Incomplete | None = None,
        no_encode: bool | None = None,
    ): ...
    def start_tls(self, read_server_info: bool = True): ...
    def do_sasl_bind(self, controls): ...
    def do_ntlm_bind(self, controls): ...
    def refresh_server_info(self) -> None: ...
    def response_to_ldif(
        self,
        search_result: Incomplete | None = None,
        all_base64: bool = False,
        line_separator: Incomplete | None = None,
        sort_order: Incomplete | None = None,
        stream: Incomplete | None = None,
    ): ...
    def response_to_json(
        self,
        raw: bool = False,
        search_result: Incomplete | None = None,
        indent: int = 4,
        sort: bool = True,
        stream: Incomplete | None = None,
        checked_attributes: bool = True,
        include_empty: bool = True,
    ): ...
    def response_to_file(self, target, raw: bool = False, indent: int = 4, sort: bool = True) -> None: ...
    @property
    def entries(self): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def grid_2d_graph(m, n, periodic: bool = False, create_using: Incomplete | None = None): ...
@_dispatch
def grid_graph(dim, periodic: bool = False): ...
@_dispatch
def hypercube_graph(n): ...
@_dispatch
def triangular_lattice_graph(
    m, n, periodic: bool = False, with_positions: bool = True, create_using: Incomplete | None = None
): ...
@_dispatch
def hexagonal_lattice_graph(
    m, n, periodic: bool = False, with_positions: bool = True, create_using: Incomplete | None = None
): ...



from _typeshed import StrOrBytesPath
from collections.abc import Callable, Iterable, Iterator
from tarfile import TarInfo
from typing import IO, Any, Protocol
from typing_extensions import TypeAlias, deprecated

from ._pygit2 import Blob, Commit, Diff, Object, Oid, Reference, Repository as _Repository, Signature, Tree, _OidArg
from .blame import Blame
from .branches import Branches
from .callbacks import CheckoutCallbacks, RemoteCallbacks, StashApplyCallbacks
from .config import Config
from .enums import (
    AttrCheck,
    BlameFlag,
    BranchType as BranchType,
    CheckoutStrategy,
    DescribeStrategy,
    DiffOption,
    MergeFavor,
    MergeFileFlag,
    MergeFlag,
    RepositoryOpenFlag,
    RepositoryState,
)
from .index import Index, IndexEntry
from .packbuilder import PackBuilder
from .references import References
from .remotes import RemoteCollection
from .submodules import Submodule, SubmoduleCollection
from .utils import _IntoStrArray

_PackDelegate: TypeAlias = Callable[[PackBuilder], None]

class _SupportsAddfile(Protocol):
    def addfile(self, tarinfo: TarInfo, fileobj: IO[bytes] | None = None) -> None: ...

class BaseRepository(_Repository):
    branches: Branches
    references: References
    remotes: RemoteCollection
    submodules: SubmoduleCollection

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...  # not meant for direct use
    def read(self, oid: _OidArg) -> tuple[int, int, bytes]: ...
    def write(self, type: int, data: bytes) -> Oid: ...
    def pack(
        self, path: StrOrBytesPath | None = None, pack_delegate: _PackDelegate | None = None, n_threads: int | None = None
    ) -> int: ...
    def __iter__(self) -> Iterator[Oid]: ...
    @deprecated("Use repo.submodules.add(...)")
    def add_submodule(self, url: str, path: str, link: bool = True, callbacks: RemoteCallbacks | None = None) -> Submodule: ...
    @deprecated("Use repo.submodules[...]")
    def lookup_submodule(self, path: str) -> Submodule: ...
    @deprecated("Use repo.submodules.init(...)")
    def init_submodules(self, submodules: Iterable[str] | None = None, overwrite: bool = False) -> None: ...
    @deprecated("Use repo.submodules.update(...)")
    def update_submodules(
        self, submodules: Iterable[str] | None = None, init: bool = False, callbacks: RemoteCallbacks | None = None
    ) -> None: ...
    def get(self, key: _OidArg, default: Object | None = None) -> Object | None: ...
    def __getitem__(self, key: _OidArg) -> Object: ...
    def __contains__(self, key: _OidArg) -> bool: ...
    @property
    def config(self) -> Config: ...
    @property
    def config_snapshot(self) -> Config: ...
    def create_reference(self, name: str, target: _OidArg, force: bool = False, message: str | None = None) -> Reference: ...
    def listall_references(self) -> list[str]: ...
    def listall_reference_objects(self) -> list[Reference]: ...
    def resolve_refish(self, refish: str) -> tuple[Commit, Reference]: ...
    def checkout_head(
        self,
        *,
        callbacks: CheckoutCallbacks | None = None,
        strategy: CheckoutStrategy | None = None,
        directory: str | None = None,
        paths: _IntoStrArray = None,
    ) -> None: ...
    def checkout_index(
        self,
        index: Index | None = None,
        *,
        callbacks: CheckoutCallbacks | None = None,
        strategy: CheckoutStrategy | None = None,
        directory: str | None = None,
        paths: _IntoStrArray = None,
    ) -> None: ...
    def checkout_tree(
        self,
        treeish: Object,
        *,
        callbacks: CheckoutCallbacks | None = None,
        strategy: CheckoutStrategy | None = None,
        directory: str | None = None,
        paths: _IntoStrArray = None,
    ) -> None: ...
    def checkout(
        self,
        refname: str | Reference | None = None,
        *,
        callbacks: CheckoutCallbacks | None = None,
        strategy: CheckoutStrategy | None = None,
        directory: str | None = None,
        paths: _IntoStrArray = None,
    ) -> None: ...
    def set_head(self, target: _OidArg) -> None: ...
    def diff(
        self,
        a: bytes | str | Oid | Blob | Tree | None = None,
        b: bytes | str | Oid | Blob | Tree | None = None,
        cached: bool = False,
        flags: DiffOption = ...,
        context_lines: int = 3,
        interhunk_lines: int = 0,
    ) -> Diff: ...
    def state(self) -> RepositoryState: ...
    def state_cleanup(self) -> None: ...
    def blame(
        self,
        path: StrOrBytesPath,
        flags: BlameFlag = ...,
        min_match_characters: int | None = None,
        newest_commit: _OidArg | None = None,
        oldest_commit: _OidArg | None = None,
        min_line: int | None = None,
        max_line: int | None = None,
    ) -> Blame: ...
    @property
    def index(self) -> Index: ...
    def merge_file_from_index(self, ancestor: IndexEntry | None, ours: IndexEntry | None, theirs: IndexEntry | None) -> str: ...
    def merge_commits(
        self,
        ours: str | Oid | Commit,
        theirs: str | Oid | Commit,
        favor: MergeFavor = ...,
        flags: MergeFlag = ...,
        file_flags: MergeFileFlag = ...,
    ) -> Index: ...
    def merge_trees(
        self,
        ancestor: str | Oid | Tree,
        ours: str | Oid | Tree,
        theirs: str | Oid | Tree,
        favor: MergeFavor = ...,
        flags: MergeFlag = ...,
        file_flags: MergeFileFlag = ...,
    ) -> Index: ...
    def merge(self, id: Oid | str, favor: MergeFavor = ..., flags: MergeFlag = ..., file_flags: MergeFileFlag = ...) -> None: ...
    @property
    def raw_message(self) -> bytes: ...
    @property
    def message(self) -> str: ...
    def remove_message(self) -> None: ...
    def describe(
        self,
        committish: str | Reference | Commit | None = None,
        max_candidates_tags: int | None = None,
        describe_strategy: DescribeStrategy = ...,
        pattern: str | None = None,
        only_follow_first_parent: bool | None = None,
        show_commit_oid_as_fallback: bool | None = None,
        abbreviated_size: int | None = None,
        always_use_long_format: bool | None = None,
        dirty_suffix: str | None = None,
    ) -> str: ...
    def stash(
        self,
        stasher: Signature,
        message: str | None = None,
        keep_index: bool = False,
        include_untracked: bool = False,
        include_ignored: bool = False,
        keep_all: bool = False,
        paths: list[str] | None = None,
    ) -> Oid: ...
    def stash_apply(
        self,
        index: int = 0,
        *,
        callbacks: StashApplyCallbacks | None = None,
        reinstate_index: bool = False,
        strategy: CheckoutStrategy | None = None,
        directory: str | None = None,
        paths: _IntoStrArray = None,
    ) -> None: ...
    def stash_drop(self, index: int = 0) -> None: ...
    def stash_pop(
        self,
        index: int = 0,
        *,
        callbacks: StashApplyCallbacks | None = None,
        reinstate_index: bool = False,
        strategy: CheckoutStrategy | None = None,
        directory: str | None = None,
        paths: _IntoStrArray = None,
    ) -> None: ...
    def write_archive(
        self, treeish: _OidArg | Tree, archive: _SupportsAddfile, timestamp: int | None = None, prefix: str = ""
    ) -> None: ...
    def ahead_behind(self, local: _OidArg, upstream: _OidArg) -> tuple[int, int]: ...
    def get_attr(
        self, path: StrOrBytesPath, name: str | bytes, flags: AttrCheck = ..., commit: Oid | str | None = None
    ) -> bool | None | str: ...
    @property
    def ident(self) -> tuple[str, str]: ...
    def set_ident(self, name: bytes | str | None, email: bytes | str | None) -> None: ...
    def revert_commit(self, revert_commit: Commit, our_commit: Commit, mainline: int = 0) -> Index: ...
    def amend_commit(
        self,
        commit: Commit | _OidArg,
        refname: Reference | str | None,
        author: Signature | None = None,
        committer: Signature | None = None,
        message: bytes | str | None = None,
        tree: Tree | _OidArg | None = None,
        encoding: str = "UTF-8",
    ) -> Oid: ...

class Repository(BaseRepository):
    def __init__(self, path: str | None = None, flags: RepositoryOpenFlag = ...) -> None: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def equivalence_classes(iterable, relation): ...
@_dispatch
def quotient_graph(
    G,
    partition,
    edge_relation: Incomplete | None = None,
    node_data: Incomplete | None = None,
    edge_data: Incomplete | None = None,
    relabel: bool = False,
    create_using: Incomplete | None = None,
): ...
@_dispatch
def contracted_nodes(G, u, v, self_loops: bool = True, copy: bool = True): ...

identified_nodes = contracted_nodes

@_dispatch
def contracted_edge(G, edge, self_loops: bool = True, copy: bool = True): ...



from collections.abc import Callable
from typing import Any

from numpy.typing import ArrayLike, NDArray

from .utils import _Seed

# TODO: This crashes stubtest. Uncomment when mypy 1.8 is released with the fix https://github.com/python/mypy/pull/16457
# @overload
# def bootstrap(
#     *args: ArrayLike,
#     n_boot: int = 10000,
#     func: str | Callable[..., Any] = "mean",
#     axis: int | None = None,
#     units: ArrayLike | None = None,
#     seed: _Seed | None = None,
# ) -> NDArray[Any]: ...
# @overload
# @deprecated("Parameter `random_seed` is deprecated in favor of `seed`")
def bootstrap(
    *args: ArrayLike,
    n_boot: int = 10000,
    func: str | Callable[..., Any] = "mean",
    axis: int | None = None,
    units: ArrayLike | None = None,
    seed: _Seed | None = None,
    random_seed: _Seed | None = None,
) -> NDArray[Any]: ...



class RedisError(Exception): ...
class AuthenticationError(RedisError): ...
class ConnectionError(RedisError): ...
class TimeoutError(RedisError): ...
class AuthorizationError(ConnectionError): ...
class BusyLoadingError(ConnectionError): ...
class InvalidResponse(RedisError): ...
class ResponseError(RedisError): ...
class DataError(RedisError): ...
class PubSubError(RedisError): ...
class WatchError(RedisError): ...
class NoScriptError(ResponseError): ...
class ExecAbortError(ResponseError): ...
class ReadOnlyError(ResponseError): ...
class NoPermissionError(ResponseError): ...
class ModuleError(ResponseError): ...
class LockError(RedisError, ValueError): ...
class LockNotOwnedError(LockError): ...
class ChildDeadlockedError(Exception): ...
class AuthenticationWrongNumberOfArgsError(ResponseError): ...
class RedisClusterException(Exception): ...
class ClusterError(RedisError): ...

class ClusterDownError(ClusterError, ResponseError):
    args: tuple[str]
    message: str
    def __init__(self, resp: str) -> None: ...

class AskError(ResponseError):
    args: tuple[str]
    message: str
    slot_id: int
    node_addr: tuple[str, int]
    host: str
    port: int
    def __init__(self, resp: str) -> None: ...

class TryAgainError(ResponseError): ...
class ClusterCrossSlotError(ResponseError): ...
class MovedError(AskError): ...
class MasterDownError(ClusterDownError): ...
class SlotNotCoveredError(RedisClusterException): ...



from _typeshed import Incomplete
from collections.abc import Iterator

class _HeapElement:
    priority: Incomplete
    element: Incomplete
    def __init__(self, priority, element) -> None: ...
    def __lt__(self, other): ...
    def __gt__(self, other): ...
    def __eq__(self, other): ...
    def __hash__(self): ...
    def __getitem__(self, indx): ...
    def __iter__(self) -> Iterator[Incomplete]: ...

class MappedQueue:
    heap: Incomplete
    position: Incomplete
    def __init__(self, data: Incomplete | None = None) -> None: ...
    def __len__(self) -> int: ...
    def push(self, elt, priority: Incomplete | None = None): ...
    def pop(self): ...
    def update(self, elt, new, priority: Incomplete | None = None) -> None: ...
    def remove(self, elt) -> None: ...



from collections.abc import Callable, Collection, Iterable, Iterator
from typing import Any, TypeVar, overload
from typing_extensions import ParamSpec

from gevent._imap import IMap, IMapUnordered
from gevent.greenlet import Greenlet
from gevent.queue import Full as QueueFull

__all__ = ["Group", "Pool", "PoolFull"]

_T = TypeVar("_T")
_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2")
_T3 = TypeVar("_T3")
_T4 = TypeVar("_T4")
_T5 = TypeVar("_T5")
_S = TypeVar("_S")
_P = ParamSpec("_P")

class GroupMappingMixin:
    def spawn(self, func: Callable[_P, _T], *args: _P.args, **kwargs: _P.kwargs) -> Greenlet[_P, _T]: ...
    # we would like to use ParamSpec for these, but since args and kwds are passed in as is
    # pyright will complain if we use _P.args/_P.kwargs, it appears to work on mypy though
    # we can probably get away with Sequence and Mapping instead of tuple and dict, but for
    # now we will be strict, just to be safe
    def apply_cb(
        self,
        func: Callable[..., _T],
        args: tuple[Any, ...] | None = None,
        kwds: dict[str, Any] | None = None,
        callback: Callable[[_T], object] | None = None,
    ) -> _T: ...
    # The ParamSpec of the spawned greenlet can differ from the one being passed in, but the return type will match
    def apply_async(
        self,
        func: Callable[..., _T],
        args: tuple[Any, ...] | None = None,
        kwds: dict[str, Any] | None = None,
        callback: Callable[[_T], object] | None = None,
    ) -> Greenlet[..., _T]: ...
    def apply(self, func: Callable[..., _T], args: tuple[Any, ...] | None = None, kwds: dict[str, Any] | None = None) -> _T: ...
    def map(self, func: Callable[[_T], _S], iterable: Iterable[_T]) -> list[_S]: ...
    def map_cb(
        self, func: Callable[[_T], _S], iterable: Iterable[_T], callback: Callable[[list[_S]], object] | None = None
    ) -> list[_S]: ...
    def map_async(
        self, func: Callable[[_T], _S], iterable: Iterable[_T], callback: Callable[[list[_S]], object] | None = None
    ) -> Greenlet[..., list[_S]]: ...
    @overload
    def imap(self, func: Callable[[_T1], _S], iter1: Iterable[_T1], /, *, maxsize: int | None = None) -> IMap[[_T1], _S]: ...
    @overload
    def imap(
        self, func: Callable[[_T1, _T2], _S], iter1: Iterable[_T1], iter2: Iterable[_T2], /, *, maxsize: int | None = None
    ) -> IMap[[_T1, _T2], _S]: ...
    @overload
    def imap(
        self,
        func: Callable[[_T1, _T2, _T3], _S],
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        /,
        *,
        maxsize: int | None = None,
    ) -> IMap[[_T1, _T2, _T3], _S]: ...
    @overload
    def imap(
        self,
        func: Callable[[_T1, _T2, _T3, _T4], _S],
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        iter4: Iterable[_T4],
        /,
        *,
        maxsize: int | None = None,
    ) -> IMap[[_T1, _T2, _T3, _T4], _S]: ...
    @overload
    def imap(
        self,
        func: Callable[[_T1, _T2, _T3, _T4, _T5], _S],
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        iter4: Iterable[_T4],
        iter5: Iterable[_T5],
        /,
        *,
        maxsize: int | None = None,
    ) -> IMap[[_T1, _T2, _T3, _T4, _T5], _S]: ...
    @overload
    def imap(
        self,
        func: Callable[_P, _S],
        iter1: Iterable[Any],
        iter2: Iterable[Any],
        iter3: Iterable[Any],
        iter4: Iterable[Any],
        iter5: Iterable[Any],
        iter6: Iterable[Any],
        /,
        *iterables: Iterable[Any],
        maxsize: int | None = None,
    ) -> IMap[_P, _S]: ...
    @overload
    def imap_unordered(
        self, func: Callable[[_T1], _S], iter1: Iterable[_T1], /, *, maxsize: int | None = None
    ) -> IMapUnordered[[_T1], _S]: ...
    @overload
    def imap_unordered(
        self, func: Callable[[_T1, _T2], _S], iter1: Iterable[_T1], iter2: Iterable[_T2], /, *, maxsize: int | None = None
    ) -> IMapUnordered[[_T1, _T2], _S]: ...
    @overload
    def imap_unordered(
        self,
        func: Callable[[_T1, _T2, _T3], _S],
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        /,
        *,
        maxsize: int | None = None,
    ) -> IMapUnordered[[_T1, _T2, _T3], _S]: ...
    @overload
    def imap_unordered(
        self,
        func: Callable[[_T1, _T2, _T3, _T4], _S],
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        iter4: Iterable[_T4],
        /,
        *,
        maxsize: int | None = None,
    ) -> IMapUnordered[[_T1, _T2, _T3, _T4], _S]: ...
    @overload
    def imap_unordered(
        self,
        func: Callable[[_T1, _T2, _T3, _T4, _T5], _S],
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        iter4: Iterable[_T4],
        iter5: Iterable[_T5],
        /,
        *,
        maxsize: int | None = None,
    ) -> IMapUnordered[[_T1, _T2, _T3, _T4, _T5], _S]: ...
    @overload
    def imap_unordered(
        self,
        func: Callable[_P, _S],
        iter1: Iterable[Any],
        iter2: Iterable[Any],
        iter3: Iterable[Any],
        iter4: Iterable[Any],
        iter5: Iterable[Any],
        iter6: Iterable[Any],
        /,
        *iterables: Iterable[Any],
        maxsize: int | None = None,
    ) -> IMapUnordered[_P, _S]: ...

# TODO: Consider making these generic in Greenlet. The drawback would be, that it
#       wouldn't be possible to mix Greenlets with different return values/ParamSpecs
#       unless you bind Grenlet[..., object], but in that case all the spawn/apply/map
#       methods become less helpful, because the return types cannot be as specific...
#       We would need higher-kinded TypeVars if we wanted to give up neither
class Group(GroupMappingMixin):
    greenlet_class: type[Greenlet[..., Any]]
    greenlets: set[Greenlet[..., Any]]
    dying: set[Greenlet[..., Any]]
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, grenlets: Collection[Greenlet[..., object]], /) -> None: ...
    def __len__(self) -> int: ...
    def __contains__(self, item: Greenlet[..., object]) -> bool: ...
    def __iter__(self) -> Iterator[Greenlet[..., object]]: ...
    def add(self, greenlet: Greenlet[..., object]) -> None: ...
    def discard(self, greenlet: Greenlet[..., object]) -> None: ...
    def start(self, greenlet: Greenlet[..., object]) -> None: ...
    def join(self, timeout: float | None = None, raise_error: bool = False) -> bool: ...
    def kill(
        self, exception: type[BaseException] | BaseException = ..., block: bool = True, timeout: float | None = None
    ) -> None: ...
    def killone(
        self,
        greenlet: Greenlet[..., object],
        exception: type[BaseException] | BaseException = ...,
        block: bool = True,
        timeout: float | None = None,
    ) -> None: ...
    def full(self) -> bool: ...
    def wait_available(self, timeout: float | None = None) -> int | None: ...

class PoolFull(QueueFull): ...

class Pool(Group):
    size: int | None
    def __init__(self, size: int | None = None, greenlet_class: type[Greenlet[..., object]] | None = None) -> None: ...
    def wait_available(self, timeout: float | None = None) -> int: ...
    def free_count(self) -> int: ...
    def start(self, greenlet: Greenlet[..., object], blocking: bool = True, timeout: float | None = None) -> None: ...
    def add(self, greenlet: Greenlet[..., object], blocking: bool = True, timeout: float | None = None) -> None: ...



from _typeshed import Incomplete

from braintree.resource import Resource

class SettlementBatchSummary(Resource):
    @staticmethod
    def generate(settlement_date, group_by_custom_field: Incomplete | None = None): ...



from typing import ClassVar

class SpringHoliday:
    include_spring_holiday: ClassVar[bool]

class SpringHolidayFirstMondayApril(SpringHoliday):
    def get_spring_holiday(self, year): ...

class SpringHolidaySecondMondayApril(SpringHoliday):
    def get_spring_holiday(self, year): ...

class SpringHolidayTuesdayAfterFirstMondayMay(SpringHoliday):
    def get_spring_holiday(self, year): ...

class SpringHolidayLastMondayMay(SpringHoliday):
    def get_spring_holiday(self, year): ...

class SpringHolidayFirstMondayJune(SpringHoliday):
    def get_spring_holiday(self, year): ...



from _typeshed import Incomplete

from antlr4.atn.ATNState import RuleStartState
from antlr4.IntervalSet import IntervalSet

class Transition:
    EPSILON: int
    RANGE: int
    RULE: int
    PREDICATE: int
    ATOM: int
    ACTION: int
    SET: int
    NOT_SET: int
    WILDCARD: int
    PRECEDENCE: int
    serializationNames: Incomplete
    serializationTypes: Incomplete
    target: Incomplete
    isEpsilon: bool
    label: Incomplete
    def __init__(self, target) -> None: ...

class AtomTransition(Transition):
    label_: Incomplete
    label: Incomplete
    serializationType: Incomplete
    def __init__(self, target, label: int) -> None: ...
    def makeLabel(self): ...
    def matches(self, symbol: int, minVocabSymbol: int, maxVocabSymbol: int): ...

class RuleTransition(Transition):
    ruleIndex: Incomplete
    precedence: Incomplete
    followState: Incomplete
    serializationType: Incomplete
    isEpsilon: bool
    def __init__(self, ruleStart: RuleStartState, ruleIndex: int, precedence: int, followState) -> None: ...
    def matches(self, symbol: int, minVocabSymbol: int, maxVocabSymbol: int): ...

class EpsilonTransition(Transition):
    serializationType: Incomplete
    isEpsilon: bool
    outermostPrecedenceReturn: Incomplete
    def __init__(self, target, outermostPrecedenceReturn: int = -1) -> None: ...
    def matches(self, symbol: int, minVocabSymbol: int, maxVocabSymbol: int): ...

class RangeTransition(Transition):
    serializationType: Incomplete
    start: Incomplete
    stop: Incomplete
    label: Incomplete
    def __init__(self, target, start: int, stop: int) -> None: ...
    def makeLabel(self): ...
    def matches(self, symbol: int, minVocabSymbol: int, maxVocabSymbol: int): ...

class AbstractPredicateTransition(Transition):
    def __init__(self, target) -> None: ...

class PredicateTransition(AbstractPredicateTransition):
    serializationType: Incomplete
    ruleIndex: Incomplete
    predIndex: Incomplete
    isCtxDependent: Incomplete
    isEpsilon: bool
    def __init__(self, target, ruleIndex: int, predIndex: int, isCtxDependent: bool) -> None: ...
    def matches(self, symbol: int, minVocabSymbol: int, maxVocabSymbol: int): ...
    def getPredicate(self): ...

class ActionTransition(Transition):
    serializationType: Incomplete
    ruleIndex: Incomplete
    actionIndex: Incomplete
    isCtxDependent: Incomplete
    isEpsilon: bool
    def __init__(self, target, ruleIndex: int, actionIndex: int = -1, isCtxDependent: bool = False) -> None: ...
    def matches(self, symbol: int, minVocabSymbol: int, maxVocabSymbol: int): ...

class SetTransition(Transition):
    serializationType: Incomplete
    label: Incomplete
    def __init__(self, target, set: IntervalSet) -> None: ...
    def matches(self, symbol: int, minVocabSymbol: int, maxVocabSymbol: int): ...

class NotSetTransition(SetTransition):
    serializationType: Incomplete
    def __init__(self, target, set: IntervalSet) -> None: ...
    def matches(self, symbol: int, minVocabSymbol: int, maxVocabSymbol: int): ...

class WildcardTransition(Transition):
    serializationType: Incomplete
    def __init__(self, target) -> None: ...
    def matches(self, symbol: int, minVocabSymbol: int, maxVocabSymbol: int): ...

class PrecedencePredicateTransition(AbstractPredicateTransition):
    serializationType: Incomplete
    precedence: Incomplete
    isEpsilon: bool
    def __init__(self, target, precedence: int) -> None: ...
    def matches(self, symbol: int, minVocabSymbol: int, maxVocabSymbol: int): ...
    def getPredicate(self): ...



from _typeshed import Incomplete

class TemplateApply:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        dry_run: Incomplete | None = None,
        org_id: Incomplete | None = None,
        stack_id: Incomplete | None = None,
        template: Incomplete | None = None,
        templates: Incomplete | None = None,
        env_refs: Incomplete | None = None,
        secrets: Incomplete | None = None,
        remotes: Incomplete | None = None,
        actions: Incomplete | None = None,
    ) -> None: ...
    @property
    def dry_run(self): ...
    @dry_run.setter
    def dry_run(self, dry_run) -> None: ...
    @property
    def org_id(self): ...
    @org_id.setter
    def org_id(self, org_id) -> None: ...
    @property
    def stack_id(self): ...
    @stack_id.setter
    def stack_id(self, stack_id) -> None: ...
    @property
    def template(self): ...
    @template.setter
    def template(self, template) -> None: ...
    @property
    def templates(self): ...
    @templates.setter
    def templates(self, templates) -> None: ...
    @property
    def env_refs(self): ...
    @env_refs.setter
    def env_refs(self, env_refs) -> None: ...
    @property
    def secrets(self): ...
    @secrets.setter
    def secrets(self, secrets) -> None: ...
    @property
    def remotes(self): ...
    @remotes.setter
    def remotes(self, remotes) -> None: ...
    @property
    def actions(self): ...
    @actions.setter
    def actions(self, actions) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import ChineseNewYearCalendar

holidays: Incomplete
workdays: Incomplete

class China(ChineseNewYearCalendar):
    shift_new_years_day: ClassVar[bool]
    include_chinese_new_year_eve: ClassVar[bool]
    extra_working_days: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    def get_calendar_holidays(self, year): ...
    def get_variable_days(self, year): ...
    def is_working_day(self, day, extra_working_days: Incomplete | None = None, extra_holidays: Incomplete | None = None): ...
    def add_working_days(
        self,
        day,
        delta,
        extra_working_days: Incomplete | None = None,
        extra_holidays: Incomplete | None = None,
        keep_datetime: bool = False,
    ): ...
    def sub_working_days(
        self,
        day,
        delta,
        extra_working_days: Incomplete | None = None,
        extra_holidays: Incomplete | None = None,
        keep_datetime: bool = False,
    ): ...



from _typeshed import Incomplete
from collections.abc import Iterator
from http.client import HTTPResponse
from json import JSONEncoder

class FluxStructure: ...

class FluxStructureEncoder(JSONEncoder):
    def default(self, obj): ...

class FluxTable(FluxStructure):
    columns: Incomplete
    records: Incomplete
    def __init__(self) -> None: ...
    def get_group_key(self): ...
    def __iter__(self): ...

class FluxColumn(FluxStructure):
    default_value: Incomplete
    group: Incomplete
    data_type: Incomplete
    label: Incomplete
    index: Incomplete
    def __init__(
        self,
        index: Incomplete | None = None,
        label: Incomplete | None = None,
        data_type: Incomplete | None = None,
        group: Incomplete | None = None,
        default_value: Incomplete | None = None,
    ) -> None: ...

class FluxRecord(FluxStructure):
    table: Incomplete
    values: Incomplete
    row: Incomplete
    def __init__(self, table, values: Incomplete | None = None) -> None: ...
    def get_start(self): ...
    def get_stop(self): ...
    def get_time(self): ...
    def get_value(self): ...
    def get_field(self): ...
    def get_measurement(self): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value): ...

class TableList(list[FluxTable]):
    def to_values(self, columns: list[str] | None = None) -> list[list[object]]: ...
    def to_json(self, columns: list[str] | None = None, **kwargs) -> str: ...

class CSVIterator(Iterator[list[str]]):
    delegate: Incomplete
    def __init__(self, response: HTTPResponse) -> None: ...
    def __iter__(self): ...
    def __next__(self): ...
    def to_values(self) -> list[list[str]]: ...



from _typeshed import Incomplete
from typing import Any

AUTH_HEADER: str
URI_QUERY: str
BODY: str
FORM_ENC_HEADERS: Any

class Client:
    refresh_token_key: str
    client_id: Any
    default_token_placement: Any
    token_type: Any
    access_token: Any
    refresh_token: Any
    mac_key: Any
    mac_algorithm: Any
    token: Any
    scope: Any
    state_generator: Any
    state: Any
    redirect_url: Any
    code: Any
    expires_in: Any
    code_verifier: str
    code_challenge: str
    code_challenge_method: str
    def __init__(
        self,
        client_id,
        default_token_placement="auth_header",
        token_type: str = "Bearer",
        access_token: Incomplete | None = None,
        refresh_token: Incomplete | None = None,
        mac_key: Incomplete | None = None,
        mac_algorithm: Incomplete | None = None,
        token: Incomplete | None = None,
        scope: Incomplete | None = None,
        state: Incomplete | None = None,
        redirect_url: Incomplete | None = None,
        state_generator=...,
        code_verifier: str | None = None,
        code_challenge: str | None = None,
        code_challenge_method: str | None = None,
        **kwargs,
    ) -> None: ...
    @property
    def token_types(self): ...
    def prepare_request_uri(self, *args, **kwargs) -> str: ...
    def prepare_request_body(self, *args, **kwargs) -> str: ...
    def parse_request_uri_response(self, *args, **kwargs) -> dict[str, str]: ...
    def add_token(
        self,
        uri,
        http_method: str = "GET",
        body: Incomplete | None = None,
        headers: Incomplete | None = None,
        token_placement: Incomplete | None = None,
        **kwargs,
    ): ...
    def prepare_authorization_request(
        self,
        authorization_url,
        state: Incomplete | None = None,
        redirect_url: Incomplete | None = None,
        scope: Incomplete | None = None,
        **kwargs,
    ): ...
    def prepare_token_request(
        self,
        token_url,
        authorization_response: Incomplete | None = None,
        redirect_url: Incomplete | None = None,
        state: Incomplete | None = None,
        body: str = "",
        **kwargs,
    ): ...
    def prepare_refresh_token_request(
        self, token_url, refresh_token: Incomplete | None = None, body: str = "", scope: Incomplete | None = None, **kwargs
    ): ...
    def prepare_token_revocation_request(
        self,
        revocation_url,
        token,
        token_type_hint: str = "access_token",
        body: str = "",
        callback: Incomplete | None = None,
        **kwargs,
    ): ...
    def parse_request_body_response(self, body, scope: Incomplete | None = None, **kwargs): ...
    def prepare_refresh_body(
        self, body: str = "", refresh_token: Incomplete | None = None, scope: Incomplete | None = None, **kwargs
    ): ...
    def create_code_verifier(self, length: int) -> str: ...
    def create_code_challenge(self, code_verifier: str, code_challenge_method: str | None = None) -> str: ...
    def populate_code_attributes(self, response) -> None: ...
    def populate_token_attributes(self, response) -> None: ...



from collections.abc import Callable
from typing import Any
from typing_extensions import TypeAlias

from exifread._types import TagDict

TAGS: TagDict

CAMERA_SETTINGS: TagDict
FOCAL_LENGTH: TagDict
SHOT_INFO: TagDict
AF_INFO_2: TagDict
FILE_INFO: TagDict

def add_one(value: int) -> int: ...
def subtract_one(value: int) -> int: ...
def convert_temp(value: int) -> str: ...

_CameraInfo: TypeAlias = dict[int, tuple[str, str, Callable[[int], Any]]]

CAMERA_INFO_TAG_NAME: str
CAMERA_INFO_5D: _CameraInfo
CAMERA_INFO_5DMKII: _CameraInfo
CAMERA_INFO_5DMKIII: _CameraInfo
CAMERA_INFO_600D: _CameraInfo
CAMERA_INFO_MODEL_MAP: dict[str, _CameraInfo]



from _typeshed import Incomplete
from collections.abc import Callable, Collection, Hashable, Iterable, Iterator, Mapping, MutableMapping
from typing import Any, ClassVar, TypeVar, overload
from typing_extensions import Self, TypeAlias

import numpy
from networkx.classes.coreviews import AdjacencyView, AtlasView
from networkx.classes.digraph import DiGraph
from networkx.classes.reportviews import DiDegreeView, NodeView, OutEdgeView

_Node = TypeVar("_Node", bound=Hashable)
_NodeWithData: TypeAlias = tuple[_Node, dict[str, Any]]
_NodePlus: TypeAlias = _Node | _NodeWithData[_Node]
_Edge: TypeAlias = tuple[_Node, _Node]
_EdgeWithData: TypeAlias = tuple[_Node, _Node, dict[str, Any]]
_EdgePlus: TypeAlias = _Edge[_Node] | _EdgeWithData[_Node]
_MapFactory: TypeAlias = Callable[[], MutableMapping[str, Any]]
_NBunch: TypeAlias = _Node | Iterable[_Node] | None
_Data: TypeAlias = (
    Graph[_Node]
    | dict[_Node, dict[_Node, dict[str, Any]]]
    | dict[_Node, Iterable[_Node]]
    | Iterable[_EdgePlus[_Node]]
    | numpy.ndarray[_Node, Any]
    # | scipy.sparse.base.spmatrix
)

class Graph(Collection[_Node]):
    node_dict_factory: ClassVar[_MapFactory] = ...
    node_attr_dict_factory: ClassVar[_MapFactory] = ...
    adjlist_outer_dict_factory: ClassVar[_MapFactory] = ...
    adjlist_inner_dict_factory: ClassVar[_MapFactory] = ...
    edge_attr_dict_factory: ClassVar[_MapFactory] = ...
    graph_attr_dict_factory: ClassVar[_MapFactory] = ...

    def to_directed_class(self) -> type[DiGraph[_Node]]: ...
    def to_undirected_class(self) -> type[Graph[_Node]]: ...
    def __init__(self, incoming_graph_data: _Data[_Node] | None = None, **attr) -> None: ...

    adj: AdjacencyView[_Node, _Node, dict[str, Incomplete]]
    name: str

    def __getitem__(self, n: _Node) -> AtlasView[_Node, _Node, dict[str, Incomplete]]: ...
    def __iter__(self) -> Iterator[_Node]: ...
    def __contains__(self, n: object) -> bool: ...
    def __len__(self) -> int: ...
    def add_node(self, node_for_adding: _Node, **attr) -> None: ...
    def add_nodes_from(self, nodes_for_adding: Iterable[_NodePlus[_Node]], **attr) -> None: ...
    def remove_node(self, n: _Node) -> None: ...
    def remove_nodes_from(self, nodes: Iterable[_Node]) -> None: ...
    nodes: NodeView[_Node]
    def number_of_nodes(self) -> int: ...
    def order(self) -> int: ...
    def has_node(self, n: _Node) -> bool: ...
    def add_edge(self, u_of_edge: _Node, v_of_edge: _Node, **attr) -> None: ...
    def add_edges_from(self, ebunch_to_add: Iterable[_EdgePlus[_Node]], **attr) -> None: ...
    def add_weighted_edges_from(
        self, ebunch_to_add: Iterable[tuple[_Node, _Node, Incomplete]], weight: str = "weight", **attr
    ) -> None: ...
    def remove_edge(self, u: _Node, v: _Node) -> None: ...
    def remove_edges_from(self, ebunch: Iterable[_EdgePlus[_Node]]) -> None: ...
    @overload
    def update(self, edges: Graph[_Node], nodes: None = None) -> None: ...
    @overload
    def update(
        self, edges: Graph[_Node] | Iterable[_EdgePlus[_Node]] | None = None, nodes: Iterable[_Node] | None = None
    ) -> None: ...
    def has_edge(self, u: _Node, v: _Node) -> bool: ...
    def neighbors(self, n: _Node) -> Iterator[_Node]: ...
    edges: OutEdgeView[_Node]
    def get_edge_data(self, u: _Node, v: _Node, default: Incomplete | None = None) -> Mapping[str, Incomplete]: ...
    def adjacency(self) -> Iterator[tuple[_Node, Mapping[_Node, Mapping[str, Incomplete]]]]: ...

    degree: DiDegreeView[_Node]

    def clear(self) -> None: ...
    def clear_edges(self) -> None: ...
    def is_multigraph(self) -> bool: ...
    def is_directed(self) -> bool: ...
    def copy(self, as_view: bool = False) -> Self: ...
    def to_directed(self, as_view: bool = False) -> DiGraph[_Node]: ...
    def to_undirected(self, as_view: bool = False) -> Graph[_Node]: ...
    def subgraph(self, nodes: Iterable[_Node]) -> Graph[_Node]: ...
    def edge_subgraph(self, edges: Iterable[_Edge[_Node]]) -> Graph[_Node]: ...
    @overload
    def size(self, weight: None = None) -> int: ...
    @overload
    def size(self, weight: str) -> float: ...
    def number_of_edges(self, u: _Node | None = None, v: _Node | None = None) -> int: ...
    def nbunch_iter(self, nbunch: _NBunch[_Node] = None) -> Iterator[_Node]: ...



from typing import ClassVar

from .core import UnitedStates

class Georgia(UnitedStates):
    include_confederation_day: ClassVar[bool]
    include_federal_presidents_day: ClassVar[bool]
    label_washington_birthday_december: ClassVar[str]
    thanksgiving_friday_label: ClassVar[str]
    def get_washington_birthday_december(self, year): ...
    def get_confederate_day(self, year): ...
    def get_robert_lee_birthday(self, year): ...
    def get_variable_days(self, year): ...



from _typeshed import Incomplete

class CreditCardGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def create(self, params: Incomplete | None = None): ...
    def delete(self, credit_card_token): ...
    def expired(self): ...
    def expiring_between(self, start_date, end_date): ...
    def find(self, credit_card_token): ...
    def forward(self, credit_card_token, receiving_merchant_id) -> None: ...
    def from_nonce(self, nonce): ...
    def update(self, credit_card_token, params: Incomplete | None = None): ...



import datetime
from _typeshed import Incomplete
from collections.abc import Callable

TZAwareDate = datetime.date
YEAR_INTERVAL: int
TIME_ZONES: Incomplete
pre_computed_equinoxes_path: Incomplete
pre_computed_solar_terms_path: Incomplete

def fromisoformat(iso): ...
def create_astronomical_data(progress: Callable[[int], int] | None = None): ...
def calculate_equinoxes(year: int, timezone: str = "UTC") -> tuple[TZAwareDate, TZAwareDate]: ...
def solar_term(year: int, degrees: int, timezone: str = "UTC") -> TZAwareDate: ...



from _typeshed import Incomplete

from braintree.resource import Resource

class PaymentMethodCustomerDataUpdatedMetadata(Resource):
    payment_method: Incomplete
    enriched_customer_data: Incomplete
    def __init__(self, gateway, attributes) -> None: ...



from collections.abc import Callable
from typing_extensions import Self

from .Image import Image

class Iterator:
    im: Image
    position: int
    def __init__(self, im: Image) -> None: ...
    def __getitem__(self, ix: int) -> Image: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> Image: ...

def all_frames(im: Image | list[Image], func: Callable[[Image], Image] | None = None) -> list[Image]: ...



from _typeshed import Incomplete

class Chunk: ...

class TagChunk(Chunk):
    tag: Incomplete
    label: Incomplete
    def __init__(self, tag: str, label: str | None = None) -> None: ...

class TextChunk(Chunk):
    text: Incomplete
    def __init__(self, text: str) -> None: ...



from _typeshed import Incomplete, StrPath
from collections.abc import Iterable, Iterator, Mapping
from typing_extensions import TypeAlias

from . import Distribution

StrIter: TypeAlias = Iterator[str]
chain_iter: Incomplete

class _Filter:
    def __init__(self, *patterns: str) -> None: ...
    def __call__(self, item: str) -> bool: ...
    def __contains__(self, item: str) -> bool: ...

class _Finder:
    ALWAYS_EXCLUDE: tuple[str, ...]
    DEFAULT_EXCLUDE: tuple[str, ...]
    @classmethod
    def find(cls, where: StrPath = ".", exclude: Iterable[str] = (), include: Iterable[str] = ("*",)) -> list[str]: ...

class PackageFinder(_Finder):
    ALWAYS_EXCLUDE: Incomplete

class PEP420PackageFinder(PackageFinder): ...
class ModuleFinder(_Finder): ...

class FlatLayoutPackageFinder(PEP420PackageFinder):
    DEFAULT_EXCLUDE: Incomplete

class FlatLayoutModuleFinder(ModuleFinder):
    DEFAULT_EXCLUDE: Incomplete

class ConfigDiscovery:
    dist: Incomplete
    def __init__(self, distribution: Distribution) -> None: ...
    def __call__(self, force: bool = False, name: bool = True, ignore_ext_modules: bool = False) -> None: ...
    def analyse_name(self) -> None: ...

def remove_nested_packages(packages: list[str]) -> list[str]: ...
def remove_stubs(packages: list[str]) -> list[str]: ...
def find_parent_package(packages: list[str], package_dir: Mapping[str, str], root_dir: StrPath) -> str | None: ...
def find_package_path(name: str, package_dir: Mapping[str, str], root_dir: StrPath) -> str: ...
def construct_package_dir(packages: list[str], package_path: StrPath) -> dict[str, str]: ...



from networkx.readwrite.json_graph.adjacency import *
from networkx.readwrite.json_graph.cytoscape import *
from networkx.readwrite.json_graph.node_link import *
from networkx.readwrite.json_graph.tree import *



from _typeshed import SupportsItems
from collections.abc import Iterable, Iterator, Mapping, Sequence
from typing import Any, ClassVar, Protocol, overload
from typing_extensions import TypeAlias

from wtforms.fields.core import Field, UnboundField
from wtforms.meta import DefaultMeta, _MultiDictLike

_FormErrors: TypeAlias = dict[str | None, Sequence[str] | _FormErrors]

# _unbound_fields will always be a list on an instance, but on a
# class it might be None, if it never has been instantiated, or
# not instantianted after a new field had been added/removed
class _UnboundFields(Protocol):
    @overload
    def __get__(self, obj: None, owner: type[object] | None = None, /) -> list[tuple[str, UnboundField[Any]]] | None: ...
    @overload
    def __get__(self, obj: object, owner: type[object] | None = None, /) -> list[tuple[str, UnboundField[Any]]]: ...

class BaseForm:
    meta: DefaultMeta
    form_errors: list[str]
    # we document this, because it's the only efficient way to introspect
    # the field names of the form, it also seems to be stable API-wise
    _fields: dict[str, Field]
    def __init__(
        self,
        fields: SupportsItems[str, UnboundField[Any]] | Iterable[tuple[str, UnboundField[Any]]],
        prefix: str = "",
        meta: DefaultMeta = ...,
    ) -> None: ...
    def __iter__(self) -> Iterator[Field]: ...
    def __contains__(self, name: str) -> bool: ...
    def __getitem__(self, name: str) -> Field: ...
    def __setitem__(self, name: str, value: UnboundField[Any]) -> None: ...
    def __delitem__(self, name: str) -> None: ...
    def populate_obj(self, obj: object) -> None: ...
    # while we would like to be more strict on extra_filters, we can't easily do that
    # without it being annoying in most situations
    def process(
        self,
        formdata: _MultiDictLike | None = None,
        obj: object | None = None,
        data: Mapping[str, Any] | None = None,
        extra_filters: Mapping[str, Sequence[Any]] | None = None,
        **kwargs: object,
    ) -> None: ...
    # same thing here with extra_validators
    def validate(self, extra_validators: Mapping[str, Sequence[Any]] | None = None) -> bool: ...
    @property
    def data(self) -> dict[str, Any]: ...
    # because of the Liskov violation in FormField.errors we need to make errors a recursive type
    @property
    def errors(self) -> _FormErrors: ...

class FormMeta(type):
    def __init__(cls, name: str, bases: Sequence[type[object]], attrs: Mapping[str, Any]) -> None: ...
    def __call__(cls, *args: Any, **kwargs: Any) -> Any: ...
    def __setattr__(cls, name: str, value: object) -> None: ...
    def __delattr__(cls, name: str) -> None: ...

class Form(BaseForm, metaclass=FormMeta):
    # due to the metaclass this should always be a subclass of DefaultMeta
    # but if we annotate this as such, then subclasses cannot use it in the
    # intended way
    Meta: ClassVar[type[Any]]
    # this attribute is documented, so we annotate it
    _unbound_fields: _UnboundFields
    def __init__(
        self,
        formdata: _MultiDictLike | None = None,
        obj: object | None = None,
        prefix: str = "",
        data: Mapping[str, Any] | None = None,
        meta: Mapping[str, Any] | None = None,
        *,
        # same issue as with process
        extra_filters: Mapping[str, Sequence[Any]] | None = None,
        **kwargs: object,
    ) -> None: ...
    # this should emit a type_error, since it's not allowed to be called
    def __setitem__(self, name: str, value: None) -> None: ...  # type: ignore[override]
    def __delitem__(self, name: str) -> None: ...
    def __delattr__(self, name: str) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.utils.backends import _dispatch

@_dispatch
def single_source_shortest_path_length(G, source, cutoff: Incomplete | None = None): ...
@_dispatch
def single_target_shortest_path_length(G, target, cutoff: Incomplete | None = None): ...
@_dispatch
def all_pairs_shortest_path_length(G, cutoff: Incomplete | None = None) -> Generator[Incomplete, None, None]: ...
@_dispatch
def bidirectional_shortest_path(G, source, target): ...
@_dispatch
def single_source_shortest_path(G, source, cutoff: Incomplete | None = None): ...
@_dispatch
def single_target_shortest_path(G, target, cutoff: Incomplete | None = None): ...
@_dispatch
def all_pairs_shortest_path(G, cutoff: Incomplete | None = None) -> Generator[Incomplete, None, None]: ...
@_dispatch
def predecessor(
    G, source, target: Incomplete | None = None, cutoff: Incomplete | None = None, return_seen: Incomplete | None = None
): ...



from _typeshed import Incomplete

class UsBankAccountGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def find(self, us_bank_account_token): ...



from typing import ClassVar

import passlib.utils.handlers as uh

class mssql2000(uh.HasRawSalt, uh.HasRawChecksum, uh.GenericHandler):
    name: ClassVar[str]
    checksum_size: ClassVar[int]
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    @classmethod
    def from_string(cls, hash): ...
    @classmethod
    def verify(cls, secret: str | bytes, hash: str | bytes) -> bool: ...  # type: ignore[override]

class mssql2005(uh.HasRawSalt, uh.HasRawChecksum, uh.GenericHandler):
    name: ClassVar[str]
    checksum_size: ClassVar[int]
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    @classmethod
    def from_string(cls, hash): ...



from _typeshed import Incomplete

class PaymentMethodGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def create(self, params: Incomplete | None = None): ...
    def find(self, payment_method_token): ...
    def update(self, payment_method_token, params): ...
    def delete(self, payment_method_token, options: Incomplete | None = None): ...
    options: dict[str, Incomplete]
    def grant(self, payment_method_token, options: Incomplete | None = None): ...
    def revoke(self, payment_method_token): ...



from _typeshed import ReadableBuffer

from serial.serialutil import SerialBase

class Serial(SerialBase):
    def open(self) -> None: ...
    @property
    def in_waiting(self) -> int: ...
    def read(self, size: int = 1) -> bytes: ...
    def write(self, b: ReadableBuffer, /) -> int | None: ...
    def reset_input_buffer(self) -> None: ...
    def reset_output_buffer(self) -> None: ...
    @property
    def cts(self) -> bool: ...
    @property
    def dsr(self) -> bool: ...
    @property
    def ri(self) -> bool: ...
    @property
    def cd(self) -> bool: ...
    def set_buffer_size(self, rx_size: int = 4096, tx_size: int | None = None) -> None: ...
    def set_output_flow_control(self, enable: bool = True) -> None: ...
    @property
    def out_waiting(self) -> int: ...
    def cancel_read(self) -> None: ...
    def cancel_write(self) -> None: ...



from typing import Any

from markdown.extensions import Extension
from markdown.treeprocessors import Treeprocessor

pygments: bool

def parse_hl_lines(expr: str) -> list[int]: ...

class CodeHilite:
    src: str
    lang: str | None
    guess_lang: bool
    use_pygments: bool
    lang_prefix: str
    pygments_formatter: Any
    options: dict[str, Any]
    def __init__(
        self,
        src: str,
        *,
        linenums: bool | None = None,
        guess_lang: bool = ...,
        css_class: str = ...,
        lang: str | None = ...,
        style: str = ...,
        noclasses: bool = ...,
        tab_length: int = ...,
        hl_lines: list[int] = ...,
        use_pygments: bool = ...,
        **options: Any,
    ) -> None: ...
    def hilite(self, shebang: bool = True) -> str: ...

class HiliteTreeprocessor(Treeprocessor):
    config: dict[str, Any]
    def code_unescape(self, text: str) -> str: ...

class CodeHiliteExtension(Extension):
    def __init__(self, **kwargs) -> None: ...

def makeExtension(**kwargs) -> CodeHiliteExtension: ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.utils.backends import _dispatch

@_dispatch
def k_edge_components(G, k): ...
@_dispatch
def k_edge_subgraphs(G, k): ...
@_dispatch
def bridge_components(G) -> Generator[Incomplete, Incomplete, None]: ...

class EdgeComponentAuxGraph:
    A: Incomplete
    H: Incomplete
    @classmethod
    def construct(cls, G): ...
    def k_edge_components(self, k) -> Generator[Incomplete, Incomplete, None]: ...
    def k_edge_subgraphs(self, k) -> Generator[Incomplete, Incomplete, None]: ...



from collections.abc import Iterable
from typing import Any, Literal, Protocol, TypeVar
from typing_extensions import TypeAlias, deprecated

from matplotlib.axes import Axes
from matplotlib.colors import Colormap
from matplotlib.typing import ColorType
from numpy.typing import ArrayLike

from ._core.typing import ColumnName, DataSource, NormSpec
from .axisgrid import FacetGrid
from .external.kde import _BwMethodType
from .utils import _DataSourceWideForm, _LogScale, _Palette, _Vector

__all__ = ["displot", "histplot", "kdeplot", "ecdfplot", "rugplot", "distplot"]

_T = TypeVar("_T")
_OneOrPair: TypeAlias = _T | tuple[_T, _T]

class _Fit(Protocol):
    def fit(self, a: ArrayLike) -> tuple[ArrayLike, ...]: ...
    def pdf(self, x: ArrayLike, *params: ArrayLike) -> ArrayLike: ...

def histplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    weights: ColumnName | _Vector | None = None,
    stat: str = "count",
    bins: _OneOrPair[str | int | ArrayLike] = "auto",
    binwidth: float | tuple[float, float] | None = None,
    binrange: _OneOrPair[tuple[float, float]] | None = None,
    discrete: bool | None = None,
    cumulative: bool = False,
    common_bins: bool = True,
    common_norm: bool = True,
    multiple: Literal["layer", "dodge", "stack", "fill"] = "layer",
    element: Literal["bars", "step", "poly"] = "bars",
    fill: bool = True,
    shrink: float = 1,
    kde: bool = False,
    kde_kws: dict[str, Any] | None = None,
    line_kws: dict[str, Any] | None = None,
    thresh: float | None = 0,
    pthresh: float | None = None,
    pmax: float | None = None,
    cbar: bool = False,
    cbar_ax: Axes | None = None,
    cbar_kws: dict[str, Any] | None = None,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    hue_norm: NormSpec = None,
    color: ColorType | None = None,
    log_scale: _LogScale | None = None,
    legend: bool = True,
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def kdeplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    weights: ColumnName | _Vector | None = None,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    hue_norm: NormSpec = None,
    color: ColorType | None = None,
    fill: bool | None = None,
    multiple: Literal["layer", "stack", "fill"] = "layer",
    common_norm: bool = True,
    common_grid: bool = False,
    cumulative: bool = False,
    bw_method: _BwMethodType = "scott",
    bw_adjust: float = 1,
    warn_singular: bool = True,
    log_scale: _LogScale | None = None,
    levels: int | Iterable[float] = 10,
    thresh: float = 0.05,
    gridsize: int = 200,
    cut: float = 3,
    clip: _OneOrPair[tuple[float | None, float | None]] | None = None,
    legend: bool = True,
    cbar: bool = False,
    cbar_ax: Axes | None = None,
    cbar_kws: dict[str, Any] | None = None,
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def ecdfplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    weights: ColumnName | _Vector | None = None,
    stat: Literal["proportion", "percent", "count"] = "proportion",
    complementary: bool = False,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    hue_norm: NormSpec = None,
    log_scale: _LogScale | None = None,
    legend: bool = True,
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def rugplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    height: float = 0.025,
    expand_margins: bool = True,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    hue_norm: NormSpec = None,
    legend: bool = True,
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def displot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    row: ColumnName | _Vector | None = None,
    col: ColumnName | _Vector | None = None,
    weights: ColumnName | _Vector | None = None,
    kind: Literal["hist", "kde", "ecdf"] = "hist",
    rug: bool = False,
    rug_kws: dict[str, Any] | None = None,
    log_scale: _LogScale | None = None,
    legend: bool = True,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    hue_norm: NormSpec = None,
    color: ColorType | None = None,
    col_wrap: int | None = None,
    row_order: Iterable[ColumnName] | None = None,
    col_order: Iterable[ColumnName] | None = None,
    height: float = 5,
    aspect: float = 1,
    facet_kws: dict[str, Any] | None = None,
    **kwargs: Any,
) -> FacetGrid: ...
@deprecated("Function `distplot` is deprecated and will be removed in seaborn v0.14.0")
def distplot(
    a: ArrayLike | None = None,
    bins: ArrayLike | None = None,
    hist: bool = True,
    kde: bool = True,
    rug: bool = False,
    fit: _Fit | None = None,
    hist_kws: dict[str, Any] | None = None,
    kde_kws: dict[str, Any] | None = None,
    rug_kws: dict[str, Any] | None = None,
    fit_kws: dict[str, Any] | None = None,
    color: ColorType | None = None,
    vertical: bool = False,
    norm_hist: bool = False,
    axlabel: str | Literal[False] | None = None,
    label: str | None = None,
    ax: Axes | None = None,
    x: ArrayLike | None = None,
) -> Axes: ...



from collections.abc import Iterable
from socket import socket as Socket
from typing import AnyStr

from .spawnbase import SpawnBase, _Logfile

__all__ = ["SocketSpawn"]

class SocketSpawn(SpawnBase[AnyStr]):
    args: None
    command: None
    socket: Socket
    child_fd: int
    closed: bool
    name: str
    use_poll: bool
    def __init__(
        self,
        socket: Socket,
        args: None = None,
        timeout: float | None = 30,
        maxread: int = 2000,
        searchwindowsize: int | None = None,
        logfile: _Logfile | None = None,
        encoding: str | None = None,
        codec_errors: str = "strict",
        use_poll: bool = False,
    ) -> None: ...
    def close(self) -> None: ...
    def isalive(self) -> bool: ...
    def send(self, s: str | bytes) -> int: ...
    def sendline(self, s: str | bytes) -> int: ...
    def write(self, s: str | bytes) -> None: ...
    def writelines(self, sequence: Iterable[str | bytes]) -> None: ...
    def read_nonblocking(self, size: int = 1, timeout: float | None = -1) -> AnyStr: ...



from _typeshed import Incomplete
from typing import ClassVar

from .core import UnitedStates

class HebrewHolidays:
    hebrew_calendars: Incomplete
    @classmethod
    def get_hebrew_calendar(cls, gregorian_year): ...
    @classmethod
    def search_hebrew_calendar(cls, gregorian_year, hebrew_month, hebrew_day): ...
    @classmethod
    def get_rosh_hashanah(cls, year): ...
    @classmethod
    def get_yom_kippur(cls, year): ...

class Florida(UnitedStates):
    include_thanksgiving_friday: ClassVar[bool]
    thanksgiving_friday_label: ClassVar[str]
    include_columbus_day: ClassVar[bool]
    include_federal_presidents_day: ClassVar[bool]

class FloridaLegal(Florida):
    FIXED_HOLIDAYS: Incomplete
    include_fat_tuesday: ClassVar[bool]
    include_lincoln_birthday: ClassVar[bool]
    include_federal_presidents_day: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    include_confederation_day: ClassVar[bool]
    include_jefferson_davis_birthday: ClassVar[bool]
    include_columbus_day: ClassVar[bool]
    columbus_day_label: ClassVar[str]
    include_election_day_every_year: ClassVar[bool]
    def __init__(self, *args, **kwargs) -> None: ...
    def get_confederate_day(self, year): ...
    def get_jefferson_davis_birthday(self, year): ...

class FloridaCircuitCourts(HebrewHolidays, Florida):
    include_federal_presidents_day: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    def get_variable_days(self, year): ...

class FloridaMiamiDade(Florida):
    include_federal_presidents_day: ClassVar[bool]
    include_columbus_day: ClassVar[bool]



from _typeshed import (
    Incomplete,
    OpenBinaryModeReading,
    OpenBinaryModeWriting,
    OpenTextModeReading,
    OpenTextModeWriting,
    SupportsWrite,
    Unused,
)
from re import Pattern
from typing import IO, Any, ClassVar, Generic, Literal, TypeVar

from docutils import TransformSpec, nodes

__docformat__: str

class InputError(OSError): ...
class OutputError(OSError): ...

def check_encoding(stream: Any, encoding: str) -> bool | None: ...
def error_string(err: BaseException) -> str: ...

_S = TypeVar("_S")

class Input(TransformSpec, Generic[_S]):
    component_type: ClassVar[str]
    default_source_path: ClassVar[str | None]
    encoding: str | None
    error_handler: str
    source: _S | None
    source_path: str | None
    successful_encoding: str | None = None
    def __init__(
        self, source: _S | None = None, source_path: str | None = None, encoding: str | None = None, error_handler: str = "strict"
    ) -> None: ...
    def read(self) -> str: ...
    def decode(self, data: str | bytes | bytearray) -> str: ...
    coding_slug: ClassVar[Pattern[bytes]]
    byte_order_marks: ClassVar[tuple[tuple[bytes, str], ...]]
    def determine_encoding_from_data(self, data: str | bytes | bytearray) -> str | None: ...
    def isatty(self) -> bool: ...

class Output(TransformSpec):
    component_type: ClassVar[str]
    default_destination_path: ClassVar[str | None]
    def __init__(
        self,
        destination: Incomplete | None = None,
        destination_path: Incomplete | None = None,
        encoding: str | None = None,
        error_handler: str = "strict",
    ) -> None: ...
    def write(self, data: str) -> Any: ...  # returns bytes or str
    def encode(self, data: str) -> Any: ...  # returns bytes or str

class ErrorOutput:
    def __init__(
        self,
        destination: str | SupportsWrite[str] | SupportsWrite[bytes] | Literal[False] | None = None,
        encoding: str | None = None,
        encoding_errors: str = "backslashreplace",
        decoding_errors: str = "replace",
    ) -> None: ...
    def write(self, data: str | bytes | Exception) -> None: ...
    def close(self) -> None: ...
    def isatty(self) -> bool: ...

class FileInput(Input[IO[str]]):
    def __init__(
        self,
        source: Incomplete | None = None,
        source_path: Incomplete | None = None,
        encoding: str | None = None,
        error_handler: str = "strict",
        autoclose: bool = True,
        mode: OpenTextModeReading | OpenBinaryModeReading = "r",
    ) -> None: ...
    def read(self) -> str: ...
    def readlines(self) -> list[str]: ...
    def close(self) -> None: ...

class FileOutput(Output):
    mode: ClassVar[OpenTextModeWriting | OpenBinaryModeWriting]
    def __getattr__(self, name: str) -> Incomplete: ...

class BinaryFileOutput(FileOutput): ...

class StringInput(Input[str]):
    default_source_path: ClassVar[str]

class StringOutput(Output):
    default_destination_path: ClassVar[str]
    destination: str | bytes  # only defined after call to write()

class NullInput(Input[Any]):
    default_source_path: ClassVar[str]
    def read(self) -> str: ...

class NullOutput(Output):
    default_destination_path: ClassVar[str]
    def write(self, data: Unused) -> None: ...

class DocTreeInput(Input[nodes.document]):
    default_source_path: ClassVar[str]



from _typeshed import Incomplete
from typing_extensions import Self

from stripe import api_requestor as api_requestor
from stripe.api_resources.abstract.api_resource import APIResource as APIResource

class CreateableAPIResource(APIResource):
    @classmethod
    def create(
        cls,
        api_key: Incomplete | None = None,
        idempotency_key: str | None = None,
        stripe_version: Incomplete | None = None,
        stripe_account: Incomplete | None = None,
        **params,
    ) -> Self: ...



from _typeshed import Incomplete
from typing import Any

MAX_ATTEMPTS: int
logger: Any

def prompt_for_confirmation(question, default: Incomplete | None = None, padding: bool = True): ...
def prompt_for_choice(choices, default: Incomplete | None = None, padding: bool = True): ...
def prompt_for_input(question, default: Incomplete | None = None, padding: bool = True, strip: bool = True): ...
def prepare_prompt_text(prompt_text, **options): ...
def prepare_friendly_prompts() -> None: ...
def retry_limit(limit=10) -> None: ...

class TooManyInvalidReplies(Exception): ...



from collections.abc import Sequence
from email import _ParamsType
from email.message import Message
from email.mime.base import MIMEBase
from email.policy import Policy

__all__ = ["MIMEMultipart"]

class MIMEMultipart(MIMEBase):
    def __init__(
        self,
        _subtype: str = "mixed",
        boundary: str | None = None,
        _subparts: Sequence[Message] | None = None,
        *,
        policy: Policy | None = None,
        **_params: _ParamsType,
    ) -> None: ...



def check_backslash(value): ...
def check_type(input_value, value_type): ...
def always_valid(input_value): ...
def validate_generic_single_value(input_value): ...
def validate_zero_and_minus_one_and_positive_int(input_value): ...
def validate_integer(input_value): ...
def validate_bytes(input_value): ...
def validate_boolean(input_value): ...
def validate_time_with_0_year(input_value): ...
def validate_time(input_value): ...
def validate_ad_timestamp(input_value): ...
def validate_ad_timedelta(input_value): ...
def validate_guid(input_value): ...
def validate_uuid(input_value): ...
def validate_uuid_le(input_value): ...
def validate_sid(input_value): ...



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class PingService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def get_ping(self, **kwargs): ...
    def get_ping_with_http_info(self, **kwargs): ...
    async def get_ping_async(self, **kwargs): ...
    def head_ping(self, **kwargs): ...
    def head_ping_with_http_info(self, **kwargs): ...
    async def head_ping_async(self, **kwargs): ...



from _typeshed import Incomplete

class NotificationEndpointBaseLinks:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        _self: Incomplete | None = None,
        labels: Incomplete | None = None,
        members: Incomplete | None = None,
        owners: Incomplete | None = None,
    ) -> None: ...
    @property
    def labels(self): ...
    @labels.setter
    def labels(self, labels) -> None: ...
    @property
    def members(self): ...
    @members.setter
    def members(self, members) -> None: ...
    @property
    def owners(self): ...
    @owners.setter
    def owners(self, owners) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from collections.abc import Iterator

from _cffi_backend import _CDataBase

from ._pygit2 import Oid, Signature

def wrap_signature(csig: _CDataBase) -> Signature: ...

class BlameHunk:
    @property
    def lines_in_hunk(self) -> int: ...
    @property
    def boundary(self) -> bool: ...
    @property
    def final_start_line_number(self) -> int: ...
    @property
    def final_committer(self) -> Signature: ...
    @property
    def final_commit_id(self) -> Oid: ...
    @property
    def orig_start_line_number(self) -> int: ...
    @property
    def orig_committer(self) -> Signature: ...
    @property
    def orig_commit_id(self) -> Oid: ...
    @property
    def orig_path(self) -> str | None: ...

class Blame:
    def __del__(self) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, index: int) -> BlameHunk: ...
    def for_line(self, line_no: int) -> BlameHunk: ...
    def __iter__(self) -> Iterator[BlameHunk]: ...



from _typeshed import Incomplete

class SecretKeys:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(self, secrets: Incomplete | None = None) -> None: ...
    @property
    def secrets(self): ...
    @secrets.setter
    def secrets(self, secrets) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import WesternCalendar

class Madagascar(WesternCalendar):
    FIXED_HOLIDAYS: Incomplete
    include_labour_day: ClassVar[bool]
    include_easter_monday: ClassVar[bool]
    include_ascension: ClassVar[bool]
    include_whit_monday: ClassVar[bool]
    include_assumption: ClassVar[bool]
    include_all_saints: ClassVar[bool]



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

__all__ = ["random_internet_as_graph"]

class AS_graph_generator:
    seed: Incomplete
    n_t: Incomplete
    n_m: Incomplete
    n_cp: Incomplete
    n_c: Incomplete
    d_m: Incomplete
    d_cp: Incomplete
    d_c: Incomplete
    p_m_m: Incomplete
    p_cp_m: Incomplete
    p_cp_cp: Incomplete
    t_m: float
    t_cp: float
    t_c: float
    def __init__(self, n, seed) -> None: ...
    G: Incomplete
    def t_graph(self): ...
    def add_edge(self, i, j, kind) -> None: ...
    def choose_peer_pref_attach(self, node_list): ...
    def choose_node_pref_attach(self, node_list): ...
    def add_customer(self, i, j) -> None: ...
    def add_node(self, i, kind, reg2prob, avg_deg, t_edge_prob): ...
    def add_m_peering_link(self, m, to_kind): ...
    def add_cp_peering_link(self, cp, to_kind): ...
    regions: Incomplete
    def graph_regions(self, rn) -> None: ...
    def add_peering_links(self, from_kind, to_kind) -> None: ...
    customers: Incomplete
    providers: Incomplete
    nodes: Incomplete
    def generate(self): ...

@_dispatch
def random_internet_as_graph(n, seed: Incomplete | None = None): ...



from _typeshed import ConvertibleToInt, Incomplete
from collections.abc import Generator, Iterable, Iterator
from datetime import datetime
from types import GeneratorType
from typing import Any, Final, Literal, NoReturn, overload
from typing_extensions import deprecated

from openpyxl import _Decodable, _VisibilityType
from openpyxl.cell import _CellValue
from openpyxl.cell.cell import Cell
from openpyxl.chart._chart import ChartBase
from openpyxl.drawing.image import Image
from openpyxl.formatting.formatting import ConditionalFormattingList
from openpyxl.workbook.child import _WorkbookChild
from openpyxl.workbook.defined_name import DefinedNameDict
from openpyxl.workbook.workbook import Workbook
from openpyxl.worksheet.cell_range import CellRange, MultiCellRange
from openpyxl.worksheet.datavalidation import DataValidation, DataValidationList
from openpyxl.worksheet.dimensions import ColumnDimension, DimensionHolder, RowDimension, SheetFormatProperties
from openpyxl.worksheet.filters import AutoFilter
from openpyxl.worksheet.page import PageMargins, PrintOptions, PrintPageSetup
from openpyxl.worksheet.pagebreak import ColBreak, RowBreak
from openpyxl.worksheet.properties import WorksheetProperties
from openpyxl.worksheet.protection import SheetProtection
from openpyxl.worksheet.scenario import ScenarioList
from openpyxl.worksheet.table import Table, TableList
from openpyxl.worksheet.views import SheetView, SheetViewList

class Worksheet(_WorkbookChild):
    mime_type: str
    BREAK_NONE: Final = 0
    BREAK_ROW: Final = 1
    BREAK_COLUMN: Final = 2

    SHEETSTATE_VISIBLE: Final = "visible"
    SHEETSTATE_HIDDEN: Final = "hidden"
    SHEETSTATE_VERYHIDDEN: Final = "veryHidden"

    PAPERSIZE_LETTER: Final = "1"
    PAPERSIZE_LETTER_SMALL: Final = "2"
    PAPERSIZE_TABLOID: Final = "3"
    PAPERSIZE_LEDGER: Final = "4"
    PAPERSIZE_LEGAL: Final = "5"
    PAPERSIZE_STATEMENT: Final = "6"
    PAPERSIZE_EXECUTIVE: Final = "7"
    PAPERSIZE_A3: Final = "8"
    PAPERSIZE_A4: Final = "9"
    PAPERSIZE_A4_SMALL: Final = "10"
    PAPERSIZE_A5: Final = "11"

    ORIENTATION_PORTRAIT: Final = "portrait"
    ORIENTATION_LANDSCAPE: Final = "landscape"

    row_dimensions: DimensionHolder[RowDimension]
    column_dimensions: DimensionHolder[ColumnDimension]
    row_breaks: RowBreak
    col_breaks: ColBreak
    merged_cells: MultiCellRange
    data_validations: DataValidationList
    sheet_state: _VisibilityType
    page_setup: PrintPageSetup
    print_options: PrintOptions
    page_margins: PageMargins
    views: SheetViewList
    protection: SheetProtection
    defined_names: DefinedNameDict
    auto_filter: AutoFilter
    conditional_formatting: ConditionalFormattingList
    legacy_drawing: Incomplete | None
    sheet_properties: WorksheetProperties
    sheet_format: SheetFormatProperties
    scenarios: ScenarioList

    def __init__(self, parent: Workbook | None, title: str | _Decodable | None = None) -> None: ...
    @property
    def sheet_view(self) -> SheetView: ...
    @property
    def selected_cell(self) -> str | None: ...
    @property
    def active_cell(self) -> str | None: ...
    @property
    def array_formulae(self) -> dict[str, str]: ...
    @property
    def show_gridlines(self) -> bool | None: ...
    @property
    def freeze_panes(self) -> str | None: ...
    @freeze_panes.setter
    def freeze_panes(self, topLeftCell: str | Cell | None = ...) -> None: ...
    def cell(self, row: int, column: int, value: str | None = None) -> Cell: ...
    @overload
    def __getitem__(self, key: int | slice) -> tuple[Cell, ...]: ...
    @overload
    def __getitem__(self, key: str) -> Any: ...  # AnyOf[Cell, tuple[Cell, ...]]
    def __setitem__(self, key: str, value: str) -> None: ...
    def __iter__(self) -> Iterator[tuple[Cell, ...]]: ...
    def __delitem__(self, key: str) -> None: ...
    @property
    def min_row(self) -> int: ...
    @property
    def max_row(self) -> int: ...
    @property
    def min_column(self) -> int: ...
    @property
    def max_column(self) -> int: ...
    def calculate_dimension(self) -> str: ...
    @property
    def dimensions(self) -> str: ...
    @overload
    def iter_rows(
        self, min_row: int | None, max_row: int | None, min_col: int | None, max_col: int | None, values_only: Literal[True]
    ) -> Generator[tuple[str | float | datetime | None, ...], None, None]: ...
    @overload
    def iter_rows(
        self,
        min_row: int | None = None,
        max_row: int | None = None,
        min_col: int | None = None,
        max_col: int | None = None,
        *,
        values_only: Literal[True],
    ) -> Generator[tuple[str | float | datetime | None, ...], None, None]: ...
    @overload
    def iter_rows(
        self,
        min_row: int | None = None,
        max_row: int | None = None,
        min_col: int | None = None,
        max_col: int | None = None,
        values_only: Literal[False] = False,
    ) -> Generator[tuple[Cell, ...], None, None]: ...
    @overload
    def iter_rows(
        self, min_row: int | None, max_row: int | None, min_col: int | None, max_col: int | None, values_only: bool
    ) -> Generator[tuple[Cell, ...], None, None] | Generator[tuple[str | float | datetime | None, ...], None, None]: ...
    @overload
    def iter_rows(
        self,
        min_row: int | None = None,
        max_row: int | None = None,
        min_col: int | None = None,
        max_col: int | None = None,
        *,
        values_only: bool,
    ) -> Generator[tuple[Cell, ...], None, None] | Generator[tuple[str | float | datetime | None, ...], None, None]: ...
    @property
    def rows(self) -> Generator[tuple[Cell, ...], None, None]: ...
    @property
    def values(self) -> Generator[tuple[_CellValue, ...], None, None]: ...
    @overload
    def iter_cols(
        self, min_col: int | None, max_col: int | None, min_row: int | None, max_row: int | None, values_only: Literal[True]
    ) -> Generator[tuple[str | float | datetime | None, ...], None, None]: ...
    @overload
    def iter_cols(
        self,
        min_col: int | None = None,
        max_col: int | None = None,
        min_row: int | None = None,
        max_row: int | None = None,
        *,
        values_only: Literal[True],
    ) -> Generator[tuple[str | float | datetime | None, ...], None, None]: ...
    @overload
    def iter_cols(
        self,
        min_col: int | None = None,
        max_col: int | None = None,
        min_row: int | None = None,
        max_row: int | None = None,
        values_only: Literal[False] = False,
    ) -> Generator[tuple[Cell, ...], None, None]: ...
    @overload
    def iter_cols(
        self, min_col: int | None, max_col: int | None, min_row: int | None, max_row: int | None, values_only: bool
    ) -> Generator[tuple[Cell, ...], None, None] | Generator[tuple[str | float | datetime | None, ...], None, None]: ...
    @overload
    def iter_cols(
        self,
        min_col: int | None = None,
        max_col: int | None = None,
        min_row: int | None = None,
        max_row: int | None = None,
        *,
        values_only: bool,
    ) -> Generator[tuple[Cell, ...], None, None] | Generator[tuple[str | float | datetime | None, ...], None, None]: ...
    @property
    def columns(self) -> Generator[tuple[Cell, ...], None, None]: ...
    def set_printer_settings(
        self, paper_size: int | None, orientation: Literal["default", "portrait", "landscape"] | None
    ) -> None: ...
    def add_data_validation(self, data_validation: DataValidation) -> None: ...
    def add_chart(self, chart: ChartBase, anchor: str | None = None) -> None: ...
    def add_image(self, img: Image, anchor: str | None = None) -> None: ...
    def add_table(self, table: Table) -> None: ...
    @property
    def tables(self) -> TableList: ...
    def add_pivot(self, pivot) -> None: ...
    # Same overload as CellRange.__init__
    @overload
    def merge_cells(
        self, range_string: str, start_row: None = None, start_column: None = None, end_row: None = None, end_column: None = None
    ) -> None: ...
    @overload
    def merge_cells(
        self,
        range_string: None = None,
        *,
        start_row: ConvertibleToInt,
        start_column: ConvertibleToInt,
        end_row: ConvertibleToInt,
        end_column: ConvertibleToInt,
    ) -> None: ...
    @overload
    def merge_cells(
        self,
        range_string: None,
        start_row: ConvertibleToInt,
        start_column: ConvertibleToInt,
        end_row: ConvertibleToInt,
        end_column: ConvertibleToInt,
    ) -> None: ...
    # Will always raise: TypeError: 'set' object is not subscriptable
    @property
    @deprecated("Use ws.merged_cells.ranges")
    def merged_cell_ranges(self) -> NoReturn: ...
    def unmerge_cells(
        self,
        range_string: str | None = None,
        start_row: int | None = None,
        start_column: int | None = None,
        end_row: int | None = None,
        end_column: int | None = None,
    ) -> None: ...
    def append(
        self,
        iterable: (
            list[Incomplete]
            | tuple[Incomplete, ...]
            | range
            | GeneratorType[Incomplete, object, object]
            | dict[int | str, Incomplete]
        ),
    ) -> None: ...
    def insert_rows(self, idx: int, amount: int = 1) -> None: ...
    def insert_cols(self, idx: int, amount: int = 1) -> None: ...
    def delete_rows(self, idx: int, amount: int = 1) -> None: ...
    def delete_cols(self, idx: int, amount: int = 1) -> None: ...
    def move_range(self, cell_range: CellRange | str, rows: int = 0, cols: int = 0, translate: bool = False) -> None: ...
    @property
    def print_title_rows(self) -> str | None: ...
    @print_title_rows.setter
    def print_title_rows(self, rows: str | None) -> None: ...
    @property
    def print_title_cols(self) -> str | None: ...
    @print_title_cols.setter
    def print_title_cols(self, cols: str | None) -> None: ...
    @property
    def print_titles(self) -> str: ...
    @property
    def print_area(self) -> str: ...
    @print_area.setter
    def print_area(self, value: str | Iterable[str] | None) -> None: ...



import enum
import sys
from collections.abc import Callable
from types import TracebackType
from typing import Any
from typing_extensions import Self

from pynput._util import AbstractListener

class Button(enum.Enum):
    unknown: int
    left: int
    middle: int
    right: int
    if sys.platform == "linux":
        button8: int
        button9: int
        button10: int
        button11: int
        button12: int
        button13: int
        button14: int
        button15: int
        button16: int
        button17: int
        button18: int
        button19: int
        button20: int
        button21: int
        button22: int
        button23: int
        button24: int
        button25: int
        button26: int
        button27: int
        button28: int
        button29: int
        button30: int
        scroll_down: int
        scroll_left: int
        scroll_right: int
        scroll_up: int
    if sys.platform == "win32":
        x1: int
        x2: int

class Controller:
    def __init__(self) -> None: ...
    @property
    def position(self) -> tuple[int, int]: ...
    @position.setter
    def position(self, position: tuple[int, int]) -> None: ...
    def scroll(self, dx: int, dy: int) -> None: ...
    def press(self, button: Button) -> None: ...
    def release(self, button: Button) -> None: ...
    def move(self, dx: int, dy: int) -> None: ...
    def click(self, button: Button, count: int = 1) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class Listener(AbstractListener):
    if sys.platform == "win32":
        WM_LBUTTONDOWN: int
        WM_LBUTTONUP: int
        WM_MBUTTONDOWN: int
        WM_MBUTTONUP: int
        WM_MOUSEMOVE: int
        WM_MOUSEWHEEL: int
        WM_MOUSEHWHEEL: int
        WM_RBUTTONDOWN: int
        WM_RBUTTONUP: int
        WM_XBUTTONDOWN: int
        WM_XBUTTONUP: int

        MK_XBUTTON1: int
        MK_XBUTTON2: int

        XBUTTON1: int
        XBUTTON2: int

        CLICK_BUTTONS: dict[int, tuple[Button, bool]]
        X_BUTTONS: dict[int, dict[int, tuple[Button, bool]]]
        SCROLL_BUTTONS: dict[int, tuple[int, int]]

    def __init__(
        self,
        on_move: Callable[[int, int], bool | None] | None = None,
        on_click: Callable[[int, int, Button, bool], bool | None] | None = None,
        on_scroll: Callable[[int, int, int, int], bool | None] | None = None,
        suppress: bool = False,
        **kwargs: Any,
    ) -> None: ...



from _typeshed import Incomplete

from stripe import api_requestor as api_requestor
from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    ListableAPIResource as ListableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
    custom_method as custom_method,
)

class CreditNote(CreateableAPIResource, ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str
    def void_credit_note(self, idempotency_key: str | None = None, **params): ...
    @classmethod
    def preview(
        cls,
        api_key: Incomplete | None = None,
        stripe_version: Incomplete | None = None,
        stripe_account: Incomplete | None = None,
        **params,
    ): ...
